dojo.provide( "com.ibm.pb.data.EnablerNavigationTreeModel" );

dojo.require("com.ibm.pb.logging.Loggable");

(function(){
	//com.ibm.mashups.enabler.model.state.NavigationStateModelFactory
	var enabler = com.ibm.mashups.enabler,
		em = enabler.model,
		state = em.state,
		accFct = state.AccessorFactory,
		spaceFct = enabler.space.Factory,
		navFct = enabler.navigation.Factory,
		lCtxt = enabler.context.Factory.getLocalizedContext(),
		dfOpP = enabler.DeferredOperation.prototype,
		opCodes = {};
	dojo.forEach(["GET","CREATE","MODIFY","DELETE"], function(c){
		this[c] = dfOpP["OPERATION_" + c];
	}, opCodes);
	
	// This module provides the API for a data model that corresponds to any data set that contains
	// parent-child relationships that can be represented as a tree.  It's exactly the same as the
	// model API that's used by dijit.Tree with some functions already implemented for ease of use.
	// As a result, any implementation of this API can be used as the model to a dijit.Tree widget
	// or any other widget that uses a data model as its backend service provider.  This data API
	// also adds some additional constraints and extension points that implementations should support
	// in addition to the model API that's used by dijit.Tree, but by no means breaks the API or
	// deviates from the required functions of the dijit model API.  One such addition is the
	// support for additional query parameters that can be specified when instantiating a model
	// that are always appended to any request to the server for data.  This allows particular instances
	// of a data model to include metadata information that would not otherwise be included by certain
	// data services that support it.
	dojo.declare( "com.ibm.pb.data.EnablerNavigationTreeModel", com.ibm.pb.logging.Loggable, {
		SAVE_CHECK: "com.ibm.mashups.builder.saveCheck",
		
		SWITCH_PAGE: "com.ibm.mashups.builder.switchPage",
		
		// root: The root item of this model
		root: null,
		
		// navigation state model
		nsm: null,
		
		// navigation model
		nm: null,
		
		// space model
		spm: null,
		
		codes: {
			HTTP_OK: 200
		},
		
		// global page support
		globalPages: [],   // the global pages that are open
		globalPageInSpace: null,  // the space in which the global pages were opened
		
		constructor: function(args) {
			dojo.safeMixin(this, args);
			if(!this.nsm) {this.nsm = state.NavigationStateModelFactory.getNavigationStateModel();}
			if(!this.nm) {this.nm = navFct.getNavigationModel();}
			if(!this.spm) {this.spm = spaceFct.getSpaceModel();}
			if(!this.delaySave) {this.delaySave = false;}
		},
		destroy: function() {
		        // summary: destroys this object, releasing connections to the store
		},
		reset: function(root) {
			// summary: Resets the root of this model to the argument.
			this.root = root;
		},
		_getRootBySpace: function(space, status, params) {
			var mname = "_getRootBySpace", lgr = this.LOGGER;
			lgr.entering(mname, arguments);
			try {
				var navDfd;
				if(space) {
					navDfd = this.nm.find(space.getRootNavigation());
				}
				else {
					navDfd = this.nm.getRoot();
				}
				navDfd.setFinishedCallback(dojo.hitch(this, function(navNode, statusCode){
					lgr.trace(mname, "Callback: ${0} ${1}", [navNode, statusCode]);
					if(navNode && !(navNode instanceof Error)) {
						this.root = navNode;
						if(params.onItem) {
							params.onItem(this.root);
						}
					}
					else if(params.onError){
						lgr.error(mname, "Error getting the root nav node in space ${0}: ${1}", [space, err]);
						params.onError(navNode);
					}
				}));
				lgr.trace(mname, "Finding root nav node in space ${0}", [space]);
				node = navDfd.start(false);
			} catch(err) {
				lgr.error(mname, "Error starting find operation for the root nav node in space ${0}: ${1}", [space, err]);
				console.error(err);
			}
			lgr.exiting(mname, arguments);
		},
		getRoot: function(onItem, onError){
			// summary:
		    //      Calls onItem with the root item for the tree
		    var mname = "getRoot", lgr = this.LOGGER;
		    lgr.entering(mname, arguments);
			var globalSpaceId = dojo.cookie(com.ibm.mm.builder.constant.cookieKey_globalPageOpenedSpaceId);
			if (globalSpaceId) {
				var id = globalSpaceId;
			} else {
				var id = accFct.getSpaceAccessor(this.nsm).getSpaceID();
			}
			var params = {onItem:onItem, onError:onError};
			if(id) {
				var dfd = this.spm.find(id);
				dfd.setFinishedCallback(dojo.hitch(this, "_getRootBySpace"), params);
				dfd.start(false);
			}
			else {
				this._getRootBySpace(null, 200, params);
			}
			lgr.exiting(mname, arguments);
		},
		mayHaveChildren: function(/*Object*/ item){
	        // summary:
	        //            Tells if an item has or may have children.  Implementing logic here
	        //            avoids showing +/- expando icon for nodes that we know don't have children.
	        //            (For efficiency reasons we may not want to check if an element actually
	        //            has children until user clicks the expando node)
	        return this.nm.hasChildren(item);
		},
		getChildren: function(/*Object*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
	        // summary:
	        // 			  Calls onComplete() with array of child items of given parent item, all loaded.
	        var mname = "getChildren", lgr = this.LOGGER;
	    	lgr.entering(mname, arguments);
	        var dfd = this.nm.getChildren(parentItem);
	        var children = [];
	        dfd.setForEachCallback(function(child){
	        	lgr.trace(mname, "Got child ${0}", [child]);
	        	if(child && !child.isHidden()) {
	        		children.push(child);
	        	}
	        });
	       	dfd.setFinishedCallback(dojo.hitch(this, function(res, status, params){
	       		lgr.trace(mname, "Finished getting children with status ${0}", [status]);
	       		if(status == this.codes.HTTP_OK) {
					// support global pages
					if (this.globalPageInSpace) {
						var parentItemSpace = com.ibm.mm.builder.utils.spaceUtil.getSpaceByNavigation(parentItem);
						if (parentItemSpace && this.globalPageInSpace == parentItemSpace.getID()) {
							for (var i in this.globalPages) {
								var inChildren = false;
								for (var j in children) {
									if (this.globalPages[i].getID() == children[j].getID()) {
										inChildren = true;
									}
								}
								if (!inChildren) {
									children.push(this.globalPages[i]);
								}
							}
						}
					}
	       			if(onComplete) {onComplete(children);}
	       		}
	       		else if(onError) {
	       			onError(res);
	       		}
	      	}));
	      	dfd.start(false);  
	      	lgr.exiting(mname, arguments); 
		},
		// =======================================================================
		// Inspecting items
		getIdentity: function(/* item */ item){
	        // summary: returns identity for an item
			if(item == null){
				return null;
			}
	        return item.getID();
		},
		getLabel: function(/*dojo.data.Item*/ item){
	        // summary: get the label for an item
			if(item == null){
				return null;
			}
	        return lCtxt.getTitle(item);
		},
		getParent: function(/*dojo.data.Item*/ item) {
			// summary: returns the parent item of this item argument
			var mname = "getParent", lgr = this.LOGGER;
		    lgr.entering(mname, arguments);
			
			if (this._isGlobalPage(item)) {
				// to support global page, return the parent of the top level pages of the space that opened the global page
				var globalSpaceId = dojo.cookie(com.ibm.mm.builder.constant.cookieKey_globalPageOpenedSpaceId);
				if (globalSpaceId) {
					var parent = com.ibm.mm.builder.utils.navigationUtil.getNavigationRootNode(globalSpaceId);
				} else {
					console.log("Error: globaPageOpenedSpaceId in cookie is missing!");
				}
			}else{
				var parent = this.nm.getParent(item).start();
			}

		    lgr.exiting(mname, [parent]);
			return parent;
		},
		// =======================================================================
		// Write interface
		newItem: function(args, parent) {
			var nm = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
			var createParams = {type: com.ibm.mashups.enabler.navigation.Constants.NAVIGATION_PAGE}
			if (args["private"] == "on") {
				createParams["private"] = true;
			} 
			var isPageTemplate = args["template"] != null;
			if (isPageTemplate) {
				createParams.pageTemplate = args["template"];
			}
			var newPage = nm.create(createParams);
			newPage.setTitle(args.title, dojo.locale);
			newPage.setTitle(args.title, ibmCfg.enablerConfig["default.locale"]);
			if (args.friendlyURL) 
				newPage.setMetaData("com.ibm.portal.friendly.name", args.friendlyURL);
			var parentId = this.getIdentity(parent);
			var parentNavNode = nm.find(parentId).start();
			nm.insert(newPage, parentNavNode);
			var content = newPage.getContent().start();
			if(!isPageTemplate) {
				content.setStaticContentURL(ibmConfig.defaultLayoutTemplateURI);
			}
			var dfd = nm.commit();
			// use verbose callback
			dfd.setOperationCallback(function(ctx){
				var evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
				// fetch request values
				var page = ctx.resource;
				var mode = ctx.mode;
				var params = ctx.parameter;
				if (ctx.request) {
					var status = ctx.request.statusCode;
				}
				// 20582 - trigger navigation refresh only after the page is created (this
				// callback is invoked not only for the page, but for the contents too)
				if(page && page instanceof com.ibm.mashups.enabler.navigation.NavigationElement && page.getID && status < 300) {
					evtSvc.broadcastEvent("com.ibm.mashups.builder.switchPage", page.getID());
					setTimeout(function(){
						// do this in a timeout in case the switchPage automatically reloads the page
						evtSvc.broadcastEvent("com.ibm.mashups.builder.spacemanager.pageCreated", page.getID());
						setTimeout(function(){
							evtSvc.broadcastEvent("com.ibm.mashups.builder.refreshNavigation");
						}, 500);
					}, 500);
				}else if(status >= 400){
					if (ctx.request) {
						var responseText = ctx.request.responseText;
					}
					var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "PageCreatorDialog");
					var statusMessage = new com.ibm.widgets.StatusMessage("error", bundle["page_dialog_error_creating_page"], responseText || "");
					dojo.publish("/message/status",[{message: statusMessage, uid: 'ibmStatusBox'}]);
				}
				if(args.callback) {
					args.callback(page, mode, status, params);
				}
			}, null, true);
			dfd.setFinishedCallback(function(page, status){
                if(status >= 400) {
					var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "PageCreatorDialog");
					var statusMessage = new com.ibm.widgets.StatusMessage("error", bundle["page_dialog_error_creating_page"], "");
					dojo.publish("/message/status",[{message: statusMessage, uid: 'ibmStatusBox'}]);
                }
				if(args.callback){
					args.callback(page, null, status);
				}
            });
			dfd.start(true);
			return newPage;
		},
		// newGlobalItem adds an item but it is temporary.  It doesn't exist in the enabler model.  This is used for global page support.
		newGlobalItem: function(args, parent){
			if(parent.getRootNavigation)
			{
				parent = com.ibm.mm.builder.utils.navigationUtil.getNavigationRootNode(parent.getID());
			}
			var newPage = com.ibm.mm.builder.utils.pageUtil.getPageNode(args.openPageId);
			newPage.loaded = false;
			newPage.created = false;			
			newPage.openPageId = args.openPageId;
			newPage.pageLoadedSpaceId = args.pageLoadedSpaceId;
			newPage.onCreate = function(){}; 
			this.globalPageInSpace = newPage.pageLoadedSpaceId;
			this.globalPages.push(newPage);
			this.newPageItem = newPage;
			this.newPageParent = parent;
			// Note: onTreeChange seems appropriate here, but it causes duplicate tabs to appear for the global page
	    	//this.onTreeChange(this.newPageItem, null, this.newPageParent);
			this._isItemCreated = true;
	        return newPage;
		},
		// Note: _createItem appears to be dead code.
		_createItem: function(args, parentInfo) {
			// summary: Creates an item
			//		All model requests must be synchronous since this must return the
			//		created item directly.
	        var title = args.title;
			
			var newPage, mix = {
				loaded: false,
				created: false
			};
			if (args.openPageId){
				// load an existing page as an new item
				newPage = this.nm.find(args.openPageId).start();
				mix = dojo.mixin(mix, {
					openPageId: args.openPageId,
					pageLoadedSpaceId: args.pageLoadedSpaceId
				});			
				this.globalPageInSpace = args.pageLoadedSpaceId;
				this.globalPage = newPage;
			}else{
				if(!this.nm.confirmCreate(args)) {
		        	// throw new Error("Not allowed");
		        	return;
		        }
		        newPage = this.nm.create(args);
		        newPage.setTitle(title, lCtxt.getPreferredLocale());
				newPage.setTitle(title, ibmCfg.enablerConfig["default.locale"]);
				newPage.setTheme(parentInfo.parent.getTheme());
			    mix.title = title;
				this.nm.insert(newPage, parentInfo.parent, args.next);			
			}
			dojo.mixin(newPage, mix);
			return newPage;
		},
		save: function(/* object */ keywordArgs) {
			// summary: Saves the model updates
			// 		Model requests can and should be asynchronous since this returns nothing
			//		is uses callbacks.
			var scope = keywordArgs.scope || dojo.global; 
	    	var utils = com.ibm.mm.builder.utils;
	    	var evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
			if (keywordArgs.openPageId){
				// just open an existing page, no need to save the page node.
				var newPage = this.nm.find(keywordArgs.openPageId).start();
				// create new items
				newPage.created = true;
				newPage.setID(newPage.getID());
				this._fireCreated(newPage);
				if(keywordArgs.onComplete) {keywordArgs.onComplete.call(scope);}
			}else{
				// create new items
		        var dfdOp = this.nm.commit();	
		        dfdOp.setOperationCallback( dojo.hitch(this, function(node){
					node.created = true;
			    	var es = com.ibm.mashups.services.ServiceManager.getService("eventService");
					es.broadcastEvent("com.ibm.mashups.builder.pageCreated", node.getID());
					node.setID(node.getID());
					this._fireCreated(node);
		    	}));
		    	dfdOp.setFinishedCallback(function(page, status){
					if(status >= 400) {
						// 4xx and 5xx HTTP errors
						var bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Theme");
						var statusMessage = new com.ibm.widgets.StatusMessage("error", bundle["dnd_move_failed"], "");
						dojo.publish("/message/status",[{message: statusMessage, uid: 'ibmStatusBox'}]);
						setTimeout(function(){
							evtSvc.broadcastEvent("com.ibm.mashups.builder.refreshNavigation");
						}, 500);
					}
		    		if(keywordArgs.onComplete) {keywordArgs.onComplete.call(scope);}
		    	});
		        dfdOp.start(false);
				if(this._isDirty){
					// update dirty items
		        	this.nm.commit().start(false);
		        	this._isDirty = false;
		        	if(keywordArgs.onComplete) {keywordArgs.onComplete(keywordArgs);}
				}			
			}
		},
		_fireCreated: function(item) {
			if(item.onCreateDefQueue && !item.onCreateDefQueue._fired) {
				try {
					item.onCreateDefQueue.fire(item);
					item.onCreateDefQueue._fired = true;
				} catch(ex) {
					// open a global page twice will lead an exception.
					console.log(ex);
				}
			}
		},
		deleteGlobalItem: function(/*Object*/args, /*item*/ item){
			// This is a fake delete item method, which only will remove global page.
			var parent = this.getParent(item);
			for (var i in this.globalPages) {
				if (this.globalPages[i].getID() == item.getID()) {
					this.globalPages.splice(i, 1);
					break;
				}
			}
	    	this.onTreeChange(null, parent, parent);
		},			
		isCreated: function(/*item*/ item) {
			// summary: Indicates whether or not the item has been created on the server and
			//		the server-generated values were properly applied to the client-side
			//		object.	NOTE: Any client-side modifications	to the values of an item for 
			// 		which isCreated returns true can potentially modify the server-generated 
			// 		values to be out-of-sync with the server.
			// 		Therefore, it's important to note that isCreated merely indicates whether
			//		or not the client-side item has a server-side resource associated with it,
			// 		and any values that the server generated for the resource (such as a unique
			return true;
		},
		addOnCreate: function(/*item*/ item, /*Function*/ callback) {
			// summary: Register callback to be called when the given item is created on the server.
			//		If the item is already created, this will call the callback immediately.
			if(this.isCreated(item)) {
				if(callback) {callback(item);}
			}
			else {
				if(!item.onCreateDefQueue) {
					item.onCreateDefQueue = new com.ibm.lang.DeferredQueue(new dojo.Deferred());
				}
				item.onCreateDefQueue.add(new dojo.Deferred().addCallback(callback));
			}
		},
		_updateChildren: function(parentItem) {
			if(parentItem) {
				this.getChildren(parentItem, dojo.hitch(this, function(children){
					this.onChildrenChange(parentItem, children);
				}));
	        }
		},
		pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ copy, /*Item*/ nextSibling, /*Boolean?*/ extra){
	        // summary:
	        //            Move an item from one parent item to another or to another position under the same parent.
	        //            Used in drag & drop.
	        // See dojo.data.api.Write for details on args.
	        var mname = "pasteItem", lgr = this.LOGGER;
	    	lgr.entering(mname, arguments);
	        if(!this.nm.confirmInsert(childItem, newParentItem, nextSibling)) {
	        	lgr.trace("confirmInsert returns false");
	        }
	        else {
	        	lgr.trace("insert into the tree model");
		        this.nm.insert(childItem, newParentItem, nextSibling);
								
		        this.onTreeChange(childItem, oldParentItem, newParentItem, nextSibling);
		        this._updateChildren(oldParentItem);
		        this._updateChildren(newParentItem);
				// When enabler correctly updates the runtime nav model, 
				// the tree model can start to use transactions for reordering
				/*if(!this.delaySave){ 
					this.save({}); 
				} else { 
					com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true); 
				}*/
	        }
	        lgr.exiting(mname, arguments);
		},
		toString: function() {
			return "[TreeModel " + this.declaredClass + "]";
		},
		_onNew: function(item, parentInfo){
			
		},
		_onDelete: function(item){
							
		},				
		_onSet: function(/* Item */ item, /*String*/ attribute, /*Object | Array*/ oldValue, /*Object | Array*/ newValue){				
			
		},
		
		_isGlobalPage: function(item){
			for (var i in this.globalPages) {
				if (this.globalPages[i].getID() == item.getID()) {
					return true;
				}
			}
			return false;
		},

		onTreeChange: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem) {
			
		},
		onChange: function(/*Item*/ item){
			
		},
		onChildrenChange: function(/*Item*/ parentItem, /*Array*/ items){
			
		}

});
	
})();


