dojo.provide("com.ibm.pb.layout.resize.ContainerResizer");

dojo.require("com.ibm.pb.layout.LayoutParser");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._base.typematic");
dojo.require("com.ibm.pb.logging.Loggable");
dojo.require("com.ibm.mashups.enabler.model.state.AccessorFactory");
dojo.require("com.ibm.mashups.enabler.model.state.NavigationStateModelFactory");

dojo.declare("com.ibm.pb.layout.resize.ContainerResizer", [com.ibm.pb.logging.Loggable, dijit._Widget, dijit._Templated],{

    templateString: '<div dojoAttachPoint="focusNode" dojoAttachEvent="onmousedown:_beginSizing" class="component-container-resizer ibmLayoutResizer lotusLeft" tabIndex="0" style="display:none;"></div>',
	layoutParser: com.ibm.pb.layout.LayoutParser,
	currContainerStyles: {left: {}, right: {}},	// the current styles set on the container runtime node
	containerStyles: {left: {}, right: {}},		// the style objects for the runtime model nodes, containerStyles.left & containerStyles.right
	runtimeNodes: {left: {}, right: {}},		// runtime skin nodes for the container, runtimeNodes.right & runtimeNodes.left
	leftNodeId: null,							// the id of the container to the left of resizer
	rightNodeId: null,							// the id of the container to the right of resizer
	eventHandles: [],							// stores the mouse event connections
	keyHandles: [],								// stores the key event connections
	height: 0, 									// the height of the resizer
	keyIncrementMap: { RIGHT_ARROW: 5,	
					   LEFT_ARROW: 5 },			// Dojo key constant name and increment to use for keyboard sizing
	defaultTimeout: 200, 						// number of milliseconds before a held key or button becomes typematic
	timeoutChangeRate: 0.9, 					// fraction of time used to change the typematic timer between events, < 1.0 fires at an increasing rate
	alwaysShow: false,							// true if the handle should always show, false to show only in edit mode
	a11yLabelId: "ibmLayoutResizerLabel",		// the id of the node used to label the resizer for accessibility
	
	constructor: function() {
		this.nlsBundle = dojo.i18n.getLocalization("com.ibm.bundles", "Theme");
		this.a11yValueText = this.nlsBundle["a11y_layout_resized"];
	},
	postCreate: function() {
		// summary: intialize variables and connect events
		var mname = "postCreate", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		
		// set the height on the resizer so it is displayed
		dojo.style(this.domNode, "height", this.height+"px");
		
		// Accessibility support using WAI-ARIA
		// - Fetch/Create a label node to be used by all resizers
		// - set the role as slider, this will give proper instruction for use
		// - set valuetext, communicates the widths of the containers associated with resizer
		var a11yLabel = dojo.byId(this.a11yLabelId);
		if(!a11yLabel){
			a11yLabel = dojo.create("div", {id: this.a11yLabelId, className: "ibmPortalHiddenText ibmHiddenText wpthemeAccess", role: "presentation", innerHTML: this.nlsBundle["layout_resizer"]}, dojo.body(), "last");
		}
		dojo.attr(this.domNode, "role", "slider");
		dojo.attr(this.domNode, "aria-labelledby", this.a11yLabelId);
		// #19024 lazy load after the resizer shows
		//var leftContainerWidth = this._getPercentageWidth(dojo.contentBox(this.layoutParser.findById(this.leftNodeId)).w); 
		//var rightContainerWidth = this._getPercentageWidth(dojo.contentBox(this.layoutParser.findById(this.rightNodeId)).w); 
		//dojo.attr(this.domNode, "aria-valuetext", dojo.string.substitute(this.a11yValueText, [leftContainerWidth, rightContainerWidth]));
		// end accessibility support
		
		if (this.alwaysShow) {
			this._showHandle();
		} else {
			this.changePageModeEvt = evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "_checkMode");
			this._checkMode(com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode());
		}
		
		// connect the listeners for the left and right arrow keys, this satisfies accessibility.		
		for(var key in this.keyIncrementMap){
			this.keyHandles[key] = dijit.typematic.addKeyListener(this.domNode,
				{charOrCode:dojo.keys[key], ctrlKey:false, altKey:false, shiftKey:false},
				this,
				function(count, node, e){
					if(node == this.domNode){
						this._changeKeySizing(e.charOrCode, this.keyIncrementMap[key]);
					}
				},
				this.timeoutChangeRate, 
				this.defaultTimeout);
		}
	},
	_hideHandle: function() {
		// summary: hides the resize handle
		if (this.domNode) {
			dojo.style(this.domNode,"display","none");
		}
	},
    _getRuntimeNodes: function() {
        if (!this.runtimeNodes.left && !this.runtimeNodes.right) {
            var runtimeModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
            this.runtimeNodes.left = runtimeModel.find(this.leftNodeId);
            this.runtimeNodes.right = runtimeModel.find(this.rightNodeId);
			
			var widths = this._getPercentageWidth(dojo.contentBox(this.runtimeNodes.left).w, dojo.contentBox(this.runtimeNodes.right).w); 
			var leftContainerWidth = widths[0];
			var rightContainerWidth = widths[1];
			dojo.attr(this.domNode, "aria-valuetext", dojo.string.substitute(this.a11yValueText, [leftContainerWidth, rightContainerWidth]));
        }
    },
	_showHandle: function() {
		// summary: shows the resize handle
		// #18844 runtimeModel is not ready in view mode
		this._getRuntimeNodes();
		var canResize = (this.runtimeNodes.left.checkStyle(this.runtimeNodes.left.getStyle()) && this.runtimeNodes.right.checkStyle(this.runtimeNodes.left.getStyle()));
		if (canResize && this.domNode) {
			this.domNode.style.display = "block";
		}
	},
	_checkMode: function(/*String*/ mode) {
		// summary: shows the resize handle if we are in page edit mode
		// mode: the page's current mode, the handle will show if mode = "edit"
		var mname = "_checkMode", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		if (mode == com.ibm.mm.iwidget.Constants.mode_edit) {
			this._showHandle();
		} else {
			this._hideHandle();
		}
	},
	_beginSizing: function(/*Event*/ e) {
		// summary: called when the user selects the resize handle with the mouse
		// e: event
		var mname = "_beginSizing", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		if (this._isSizing) {   // return if the handle is already dragging
			return false; 
		} else {
			this._isSizing = true;
		}
		
		// set the cursor on the body
		dojo.style(dojo.body(), {cursor:dojo.style(this.domNode).cursor});
		this.startX = e.clientX;

		this._getStartValues();	
		this.orignalWidgetsSize=this._getWidgetsSizing();

		this.eventHandles.push(dojo.connect(document, "onmousemove", this, "_changeSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseup", this, "_endSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseleave", this, "_endSizing")); 
		
		e.preventDefault();		
	},
	_changeKeySizing:function(/*Event*/ key, /*int*/increment){
		// summary: called when the user uses the arrow keys on the resize handle
		// key: key being pressed
		var mname = "_changeKeySizing", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this._getStartValues();		
		
		var leftActualWidth,
			rightActualWidth;
		
		if(key === dojo.keys.RIGHT_ARROW){
			leftActualWidth = this.leftStartWidth + increment;			
			rightActualWidth = this.rightStartWidth - increment;	
		}else if(key === dojo.keys.LEFT_ARROW){			
			leftActualWidth = this.leftStartWidth - increment;
			rightActualWidth = this.rightStartWidth + increment;	
		}
		
		this._updateContainersWidth(leftActualWidth, rightActualWidth);

		// update the text for accessibility
		dojo.attr(this.domNode, 
				"aria-valuetext", 
				dojo.string.substitute(this.a11yValueText, [this.currContainerStyles.left.width, this.currContainerStyles.right.width]));
		
		// save the changes
		this.runtimeNodes.left.setStyle(this.currContainerStyles.left);
		this.runtimeNodes.right.setStyle(this.currContainerStyles.right);
	},
	_changeSizing: function(/*Event*/ e) {
		// summary: called when the user moves the resize handle
		// e: DOM event		
		var leftActualWidth,
			rightActualWidth,
			dx;
		
		if (this.startX > e.clientX){
			// dragging right to left
			dx = this.startX - e.clientX;
			leftActualWidth = this.leftStartWidth - dx;
			rightActualWidth = this.rightStartWidth + dx;
		}else{
			// dragging left to right		
			dx = e.clientX - this.startX;			
			leftActualWidth = this.leftStartWidth + dx;
			rightActualWidth = this.rightStartWidth - dx;	
		}
		
		var isUpdated = this._updateContainersWidth(leftActualWidth, rightActualWidth);
		
		if(!isUpdated){
			// If the style was not updated by the filter, 
			// then grab the values in the DOM to make sure 
			// what is displayed on screen matches what is saved.
			var widths = this._getPercentageWidth(dojo.contentBox(this.layoutParser.findById(this.leftNodeId)).w, dojo.contentBox(this.layoutParser.findById(this.rightNodeId)).w); 
			this.currContainerStyles.left.width = widths[0];
			this.currContainerStyles.right.width = widths[1];
		}
		
		e.preventDefault();
	},
	_endSizing: function(/*Event*/ e) {
		// summary: called when the user stops moving the resize handle, does clean up and then sets the style on the skin
		// e: DOM event
		var mname = "_endSizing", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		this._isSizing = false;
		
		// cleanup: reset cursor, set the page as dirty, and disconnect drag handlers
		dojo.style(dojo.body(),{cursor:"default"});
		com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setDirty(true);
		while (this.eventHandles.length > 0) {
			dojo.disconnect(this.eventHandles.pop());
		}
		
		// update the text for accessibility
		dojo.attr(this.domNode, 
				"aria-valuetext", 
				dojo.string.substitute(this.a11yValueText, [this.currContainerStyles.left.width, this.currContainerStyles.right.width]));
		
		// save the changes
		this.runtimeNodes.left.setStyle(this.currContainerStyles.left);
		this.runtimeNodes.right.setStyle(this.currContainerStyles.right);
		this._setContainersWidth();
		this._sendResizeEvents();
		this.onEndSizing(this.containerStyles);
	},
	_sendResizeEvents:function(){
		this._sendResizeEventsOnContainer(this._getWidgetsSizing());
	},
	_sendResizeEventsOnContainer:function(widgetSizings){
		var evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		for(var id in widgetSizings){
			evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + id, this._updateWidgetSize(widgetSizings[id]));
		}
	},
	_updateWidgetSize:function(newSize){
		var result = {"widgetId":newSize.widgetId};
		result.newHeight= newSize.newContentHeight;
		result.newWidth= newSize.newContentWidth;
		if(!this.orignalWidgetsSize) return result;
		var oldSize= this.orignalWidgetsSize[newSize.widgetId];
		if(!!oldSize){
			//container resizer only update widgets' width, not height.
			//Widget height may changed while doing container resize. This is caused by the automatically generated horizontal scroll bar.
			if (oldSize.newContentHeight > newSize.newContentHeight) {
				result.newHeight = oldSize.newContentHeight;
			}
			var widthGap = (newSize.newMarginWidth - newSize.newContentWidth) -(oldSize.newMarginWidth - oldSize.newContentWidth);
			if(widthGap >0){//While resizing, we do not change the widget style, the gap between margin & content should be fixed, 
							//widthGap means there may be a vertical scroll bar show up during resize, but this scrollbar may not be needed if the widget
							//can handle properly for the size change.
				if(newSize.newContentWidth +widthGap <  newSize.newMarginWidth ){//I don't know any situation that this can be false, just in case.
					result.newWidth =newSize.newContentWidth +widthGap;
				}
			}
			
						
		}
		return result;
	},
	_getWidgetsSizing:function(){
		return dojo.mixin(this._getWidgetsSizingByContainer(this.leftNodeId),this._getWidgetsSizingByContainer(this.rightNodeId));
	},
	
	_getWidgetsSizingByContainer:function(containerId){
		var widgetsSizing={};
		dojo.forEach(this.layoutParser.findControls(this.layoutParser.findById(containerId)),function(control){
			var id = this.layoutParser.extractID(control);
			var div = dojo.query(".lotusWidgetBody",control)[0]; 
			var contentBox =dojo.contentBox(div);
			var marginBox =dojo.marginBox(div);
			var changes = {"widgetId":id};
			changes.newContentWidth = contentBox.w;
			changes.newContentHeight = contentBox.h;
			changes.newMarginWidth = marginBox.w;
			changes.newMarginHeight = marginBox.h;
			widgetsSizing[id]=changes;
		},this);
		return widgetsSizing;
	},
	_setContainersWidth:function(){
		var pid = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
        var layoutAccessor =  com.ibm.mashups.enabler.model.state.AccessorFactory.getLayoutAccessor(navStateModel, pid);

        // if there is no template url associated with this layout navstate
        // then get it from the page instance
        // and set it here
        if (!layoutAccessor.getTemplateURL()) {
            var navNode = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().find(pid).start(true);
            var content = navNode.getContent().start(true);
            var templateUrl = content.getStaticContentURL();

            layoutAccessor.setTemplateURL(templateUrl)
        }

		this._setContainerWidth(layoutAccessor,this.leftNodeId, this.rightNodeId);
		navStateModel.commit().start();
	},
	_setContainerWidth:function(layoutAccessor, cidLeft, cidRight){
	    var widths = this._getPercentageWidth(dojo.contentBox(this.layoutParser.findById(cidLeft)).w, dojo.contentBox(this.layoutParser.findById(cidRight)).w);
		var containerAcc = layoutAccessor.getContainerAccessor(cidLeft);
		containerAcc.setSize( widths[0] );
		containerAcc = layoutAccessor.getContainerAccessor(cidRight);
		containerAcc.setSize( widths[1] );
	},
	onEndSizing: function(/*Object*/ styles) {
		// summary: called when the layout has been resized
		// style: an object storing style objects for both containers that were resized, styles.left and styles.right
		//	        a styles object consists of these properties {width, height, left, right}, i.e. styles.left.width
		//			if a style property for the width or height exists, it has s a string format of "XXXXpx"
		//			styles.left has the styles for the container to the left of the resizer
		//			styles.right has the styles for the container to the right of the resizer
	},	
	_getStartValues:function(){		
		// summary: retrieves the values required when starting a resize action
		var mname = "_getStartValues", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
        this._getRuntimeNodes();
		this.currContainerStyles.left = this.runtimeNodes.left.getStyle();
		this.currContainerStyles.right = this.runtimeNodes.right.getStyle();
	
		this.leftStartWidth = dojo.contentBox(this.layoutParser.findById(this.leftNodeId)).w; 
		this.rightStartWidth = dojo.contentBox(this.layoutParser.findById(this.rightNodeId)).w;
				
		/* In Safari the percentages translate to be 1 pixel smaller by dojo.contentBox
		 * than what is currently displayed on screen, 1 pixel is added to maintain consistency, 
		 * otherwise the containers shrink as you use the arrow keys */
		if(dojo.isSafari){
			this.leftStartWidth++;
			this.rightStartWidth++;
		}
	},
	_updateContainersWidth: function(leftWidth, rightWidth){
		// summary: update the width of the containers in the DOM and 
		//			merge the new width with the current runtime style object
		// leftWidth: the new width of the left container
		// rightWidth: the new width of the right container
		var mname = "_updateContainersWidth", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		// convert the width to a percentage
		var widths = this._getPercentageWidth(leftWidth, rightWidth);
		this.containerStyles.left.width = widths[0];
		this.containerStyles.right.width = widths[1];
	
		// check to see if the width is to be updated
		var isUpdated = this.runtimeNodes.left.checkStyle(this.containerStyles.left) && this.runtimeNodes.right.checkStyle(this.containerStyles.right);
		
		if(isUpdated){			
	        // mixin the new widths to the current styles set on the runtime node
			dojo.mixin(this.currContainerStyles.left, this.containerStyles.left);
			dojo.mixin(this.currContainerStyles.right, this.containerStyles.right);
		
			// set the percentage width on the container DOM node
			dojo.style(this.layoutParser.findById(this.leftNodeId), "width", this.containerStyles.left.width);		
			dojo.style(this.layoutParser.findById(this.rightNodeId), "width", this.containerStyles.right.width);		
		}
		
		return isUpdated;
	},
	_getPercentageWidth: function(containerWidth1, containerWidth2){
		// summary: calculates the container width as a percentage of the total layout
		var layoutWidth = dojo.contentBox(this.layoutParser.getRootNode()).w + 1;
		// calculate the first width
		var ret = (containerWidth1 / layoutWidth) * 100;
		// now calculate the second width
		var ret2 = (containerWidth2 / layoutWidth) * 100;
		// calculate the target total percentage
		var target = 95.75;
		if (this.currContainerStyles.left.width && this.currContainerStyles.right.width) {
			// after already calculated the first time, reuse from then on so nothing ever shrinks or grows
			target = parseFloat(this.currContainerStyles.left.width) + parseFloat(this.currContainerStyles.right.width);
		} else {
			// calculate the first time
			target = ((containerWidth1 + containerWidth2) / layoutWidth) * 100;
		}
		// adjust if current total percentage is different from target total percentage
		var adjustment = ((ret+ret2) - target) / 2;
		ret -= adjustment;
		ret2 -= adjustment;

		return [ret+"%", ret2+"%"];
	},
	destroy: function() {
		var mname = "destroy", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		// disconnect all events
		for(var key in this.keyHandles){
			for(var i=0; i<this.keyHandles[key].length; i++){
				dojo.disconnect(this.keyHandles[key][i]);
			}
		}
		while (this.eventHandles.length > 0) {
			dojo.disconnect(this.eventHandles.pop());
		}
		this.inherited(arguments);
		return;
	}
});
