// @author Laura
// This module is a handle that can be attached to a widget skin and used to drag the widge to a different size

dojo.provide("com.ibm.pb.layout.resize.WidgetResizer");
dojo.require("com.ibm.pb.layout.LayoutParser");
dojo.require("com.ibm.mashups.enabler.model.state.NavigationStateModelFactory");
dojo.require("com.ibm.mashups.enabler.model.state.AccessorFactory");
dojo.require("com.ibm.mashups.enabler.model.state.WidgetAccessor");
dojo.declare("com.ibm.pb.layout.resize.WidgetResizer", [dijit._Widget, dijit._Templated],{


	
	widgetId: null, 		// id of the widget
	controlId: "",			// the id of the given layout control
	containerId: "",		// the id of the container in which the control resides
	skinNode: null,			// a reference to the control's modifiable runtime skin node which corresponds to the given control
	alwaysShow: false,		// true if the handle should always show, false to show only in edit mode, PLEASE NOTE: you must provide a mechanism to save the resizing in view mode, it is only automatic in edit mode
	controlStyle: {},		// the current style on the skin object from the DOM in dojo.style format
	widgetStyle: {},		// the current style on the widget node from the DOM in dojo.style format
	point: {x:0,y:0},		// the {x,y} coordinates of the dragging handle point
	positionAttrs: {"width":null,"height":null,"top":null,"left":null,"position":"static"},	// list of style attribute names that have to do with position
	templateString: '<div>' +
						'<div dojoAttachPoint="handle" dojoAttachEvent="onmousedown:_beginSizing" class="ibmWidgetResizer lotusRight" style="display:none;"></div>' +
						'<div style="clear: both;"></div>' +
					'<div>',
	
	postCreate: function() {
		// summary: intialize variables and connect events
		this.evtSvc = com.ibm.mashups.services.ServiceManager.getService("eventService");
		this.widgetMovedHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetMoved", this, "_clearStyle");
		
		//This is used to avoid losing position when switching to freeform layout and then dnd a widget
		this.initWidgetHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.initWidgets", this, "_initWidgetsInFreeformLayout");
		if (this.alwaysShow) {
			// if the handle should always be shown, make it visible now
			this._showHandle();
		} else {
			// if the handle should only be shown in page edit mode, attach to the changePageMode event to we know when to show it
			this.changeModeHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "_checkMode");
            if (this.forceShowHandle) {
                this._checkMode(com.ibm.mm.iwidget.Constants.mode_edit);
            }
            else {
                this._checkMode(com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode());
            }
		}
	},
	
	destroy: function() {
		// summary: destoys the nodes and connections associated with this dijit
		this.evtSvc.unsubscribeEvent(this.widgetMovedHandler, null);
		this.evtSvc.unsubscribeEvent(this.initWidgetHandler, null);
		this.evtSvc.unsubscribeEvent(this.changeModeHandler, null);
		this.widgetId = null;
		this.inherited(arguments);
	},
	
	/** GET/SET STYLE FROM/ON THE SERVER **/
	
	setStyle: function(/*Object*/ style) {
		// summary: called to save a new skin style to the server
		// style: a style object
		//		  if it exists, height is found at style.height and has a string format of "XXXXpx"
		//		  if it exists, width is found at style.width and has a string format of "XXXXpx"
		// a widget instance can be retrieved from the Widget Model API using the ID: this.widgetId
		style = style || {};
		this.skinNode.setStyle(style);
	},
	
	getStyle: function() {
		// returns the style object for the current node from wherever it is stored server-side
		return this.skinNode.getStyle();
	},
	
	_setWidgetStyle: function(/*String*/ style) {
		// summary: along with _setWidgetStyleContinue() sets the wrapper-css-style metadata on the control that will get applied to its widget DOM node
		// style: a string that represents the inline style for the widget DOM node
		var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var layoutModel = navModel.getLayoutModel(pageId);
		var control = layoutModel.find(this.controlId);
		var me = this;
		control.setFinishedCallback( function(control, status) {
			me._setWidgetStyleContinue(control, status, style);
		}, style);
		control.start(false);
	},
	
	_setWidgetStyleContinue: function(control, status, style) {
		// summary: along with _setWidgetStyle() sets the wrapper-css-style metadata on the control that will get applied to its widget DOM node
		// control: a layout control on which to set the metadata
		// status: the HTTP status result from retrieving the control
		// style: a string that represents the inline style for the widget DOM node
		if (status == 200) {
			control.setMetaData("widget-body-height", style);
		} else {
			console.error("Could not find control when setting wrapper-css-style metadata");
		}
	},
	
	/** RESIZE DOM EVENT FUNCTIONS **/
	
	_beginSizing: function(/*Event*/ e) {
		// summary: called when the user selects the resize handle with the mouse
		// e: DOM event
		if (this._isSizing) {   // return if the handle is already dragging
			return false; 
		} else {
			this._isSizing = true;
		}
		
		// set the drag cursor on the body
		dojo.style(dojo.body(), {cursor:dojo.style(this.handle).cursor});
		
		// store these values so they can be dynamically updated as the handle is dragged: 	
		//				x and y coordinates of the drag handle
		//				the style on the skin/control node
		//				the style on the widget body node
		this.point = {'x':e.clientX, 'y':e.clientY};
		// results of _getDomPositionStyle are mixed together in case the left/height/width attributes are not included in 
		//		the runtime skin style from getStyle() but are present in the DOM and stored elsewhere
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		this.controlStyle  = dojo.mixin(this._getDomPositionStyle(controlNode), this.getStyle()); 
		var widgetNode = dojo.byId(this.widgetId).parentNode;
		this.widgetStyle = this._getDomPositionStyle(widgetNode);
		this.oldSize= this._getWidgetSize();
		
		// connect to events for dragging the mouse
		this.eventHandles = []; 
		this.eventHandles.push(dojo.connect(document, "onmousemove", this, "_changeSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseup", this, "_endSizing")); 
		this.eventHandles.push(dojo.connect(document, "onmouseleave", this, "_endSizing")); 

		e.preventDefault();	
	},
	
	_getWidgetSize:function(){
		var size={};
		var control = com.ibm.pb.layout.LayoutParser.findById(this.controlId);
		var div = dojo.query(".lotusWidgetBody",control)[0]; 
		var contentBox =dojo.contentBox(div);
		var marginBox =dojo.marginBox(div);
		size.newContentWidth = contentBox.w;
		size.newContentHeight = contentBox.h;
		size.newMarginWidth = marginBox.w;
		size.newMarginHeight = marginBox.h;
		return size;
	},

	_changeSizing: function(/*Event*/ e) {
		// summary: called when the user moves the resize handle
		// e: DOM event

		// calculate the dy/dx of the drag handle => calculate the new height and width of the control
		var dx = this.point.x - e.clientX;  this.point.x -= dx;
		var dy = this.point.y - e.clientY; 	this.point.y -= dy;
		var oldHeight = this.controlStyle.height;
		this._updatePosition(this.controlStyle, dx, dy);  // changes the height, width and left attributes to incorporate dy/dx
		
		// filter the style and set the resulting style on the skin in the DOM
		// need to create a new style object instead of using this.controlStyle directly to avoid errors in IE
		// the IE errors happen because sometimes this.controlStyle has other attributes in it that are out of our control
		this.skinNode.checkStyle(this.controlStyle);  // NOTE: checkStyle will update the this.controlStyle object
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		var style = {};
		style.height = this.controlStyle.height ? this.controlStyle.height : null;
		style.width = this.controlStyle.width ? this.controlStyle.width : null;
		style.left = this.controlStyle.left ? this.controlStyle.left : null;
		dojo.style(controlNode, style);
		
		// change height on the widget body to reflect changes in the skin height
		// this is to make sure the widget body takes up the full height available to it in the skin
		if (this.controlStyle.height) {
			var dy = parseInt(oldHeight) - parseInt(this.controlStyle.height);
			this._updatePosition(this.widgetStyle, dx, dy);  // changes the height attribute to incorporate dy/dx
			var widgetNode = dojo.byId(this.widgetId).parentNode;
			dojo.style(widgetNode, "height", this.widgetStyle.height);
		}

		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e) {
        // summary: called when the user stops moving the resize handle, does clean up and then sets the style on the skin
		// e: DOM event

	    var currentPage = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage();

		this._isSizing = false;
		
		// cleanup: reset cursor, set the page as dirty, and disconnect drag handlers
		dojo.style(dojo.body(),{cursor:"default"});
		currentPage.setDirty(true);
		while (this.eventHandles.length > 0) {
			dojo.disconnect(this.eventHandles.pop());
		}

		// save the change in skin/control size and widget height
		this.setStyle(this.controlStyle);
		if (this.controlStyle.height) {
			// a height is set on the widget body as well as the skin/control to make sure the widget fills up the entire available height inside the skin
			this._setWidgetStyle(this.widgetStyle.height);
		}

        // if there is no template url
        // associated with this layout navstate
        // then get it from the page instance
        // and set it here
        var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
        var layoutAccessor =  com.ibm.mashups.enabler.model.state.AccessorFactory.getLayoutAccessor(navStateModel, currentPage.getID());

        if (!layoutAccessor.getTemplateURL()) {
            var navNode = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel().find(currentPage.getID()).start(true);
            var content = navNode.getContent().start(true);
            var templateUrl = content.getStaticContentURL();

            layoutAccessor.setTemplateURL(templateUrl)
        }
        
		// save the new control size in the NavState
		var navStateModel = com.ibm.mashups.enabler.model.state.NavigationStateModelFactory.getNavigationStateModel();
		var accessorFactory = com.ibm.mashups.enabler.model.state.AccessorFactory;
		var widgetAccessor = accessorFactory.getWidgetAccessor(navStateModel, this.widgetId);
		widgetAccessor.setSize(this.controlStyle.width, this.controlStyle.height);
		navStateModel.commit().start();

		// broadcast the widget change size event to tell the widget it can reflow since its size has changed
		// create a new changes object instead of just using {height:this.controlStyle.height,width:this.controlStyle.width} 
		//			because if either height or width is null, errors are thrown
		var box = this._getBoxWithScroll();
		var changes = {"widgetId":this.widgetId};
		if (this.controlStyle.width) {
			changes.newWidth = parseInt(this.controlStyle.width);
		}else {
			changes.newWidth=box.w;
		}
		if (this.controlStyle.height) {
			changes.newHeight = parseInt(this.widgetStyle.height);
		}else {
			changes.newHeight = box.h;
		}
		this.evtSvc.publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + this.widgetId, changes);
		this.evtSvc.publishEvent("com.ibm.mashups.builder.resizeWidget", changes);  // needed for freeform layout
	},
	
	_getSizeWithScroll:function(oldSize,newSize,dimension){
		var gap = (newSize["newMargin"+dimension] - newSize["newContent"+dimension]) -(oldSize["newMargin"+dimension] - oldSize["newContent"+dimension]);
		if(gap >0){//While resizing, we do not change the widget style, the gap between margin & content should be fixed, 
						//gap means there may be a scroll bar show up during resize, but this scrollbar may not be needed if the widget
						//can handle properly for the size change.
			if(newSize["newContent"+dimension] +gap < newSize["newMargin"+dimension] ){//I don't know any situation that this can be false, just in case.
				return newSize["newContent"+dimension] +gap;
			}
		}
		return newSize["newContent"+dimension];
	},
	
	_getBoxWithScroll:function(){
		var newSize = this._getWidgetSize();
		var result = {h:newSize.newContentHeight,w:newSize.newContentWidth};
		if(!this.oldSize) return result;
		result.h= this._getSizeWithScroll(this.oldSize,newSize,"Height");
		result.w= this._getSizeWithScroll(this.oldSize,newSize,"Width");
		return result;
	},
	
	/** HANDLE HELPERS **/
	
	_hideHandle: function() {
		// summary: hides the resize handle
		if(this.handle) {
			dojo.style(this.handle,'display',"none");
		}
	},
	
	_showHandle: function() {
		// summary: shows the resize handle
		// #18844 runtimeModel is not ready in view mode
		if (!this.skinNode) {
			var runtimeModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
			this.skinNode = runtimeModel.find(this.controlId);
			//#19026 lazy find when first time show the resize handle
			var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
			this.containerId = com.ibm.pb.layout.LayoutParser.extractID(controlNode.parentNode);
		}
		var canResize = this.skinNode.checkStyle(this.getStyle());
		if (canResize && this.handle) {
			dojo.style(this.handle,'display',"block");
		}
	},
	
	/** FUNCTIONS FOR ATTACHING TO ENABLER EVENTS **/
	
	_checkMode: function(/*String*/ mode) {
		// summary: shows the resize handle if we are in page edit mode
		// mode: the page's current mode, the handle will show if mode = "edit"
		if (mode == com.ibm.mm.iwidget.Constants.mode_edit) {
			this._showHandle();
		} else {
			this._hideHandle();
		}
	},
	_initWidgetsInFreeformLayout: function(iEvent){
		this.containerId = iEvent.containerId;
	},
	_clearStyle: function(/*string*/ id) {
		// summary: clears out styles that have been set on the skin and widget nodes on the server and in the DOM
		//			ONLY triggered if the control has been moved to another container
		// id: id for which the styles should be cleared
		if (this.controlId != id) {
			// not me
			return;
		}
		
		var controlNode = com.ibm.pb.layout.LayoutParser.findById(this.controlId);	
		var parentNode = controlNode.parentNode;

		var newContainerId = com.ibm.pb.layout.LayoutParser.extractID(parentNode);

		// if the node was dropped to a freeform container, skip clearing styles (similar to init)
		if (dojo.hasClass(parentNode, "ibmFreeform")) {
		    this.containerId = newContainerId;
		    return;
		}

		if (newContainerId != this.containerId) {
			this.containerId = newContainerId;
			// clear the position attributes from the current style
			var noPositionStyle = this._removePositionFromStyle(this.getStyle());
			// save the control and widget style to the server
			this.setStyle(noPositionStyle);
			this._setWidgetStyle("");
			// clear the styles in the DOM too
			var w = dojo.byId(this.widgetId);
			if(w) { 
				if(dojo.isSafari) {
					dojo.style(w.parentNode, "height", null);
				} else {
					w.parentNode.removeAttribute("style");
				}
			}
			dojo.style(controlNode, this.positionAttrs);
			// clear the runtime model cache for this node
			var rsModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
			rsModel.invalidate(this.skinNode);
		}
	},
	
	/** HELPERS TO MANIPULATE STYLE OBJECTS **/
	
	_removePositionFromStyle: function(/*Object*/ style) {
		// summary: removes all the DOM attributes having to do with position from a given style object
		//			for example, this is done when a widget is moved into a different container and needs to be reset
		// style: the style object to manipulate
		for (var i in this.positionAttrs) {
			if (style[i]) {
				delete style[i];
			}
		}
	},
	
	_getDomPositionStyle: function(/*DOMNode*/ node) {
		// summary: returns a style object with only height/width/left?/top? attributes
		//			style attributes are retrieved from the DOM
		// node: the DOM node for which to get the style
		var dBox = dojo.contentBox(node);
		var dStyle = dojo.style(node);
		var ret = {};
		ret.height = dStyle.height;
		if (ret.height == "auto") {
			ret.height = dBox.h + "px";
		}
		ret.width = dStyle.width;
		if (ret.width == "auto") {
			ret.width = dBox.w + "px";
		}
		if (dStyle.left != "auto") {
			ret.left = dStyle.left;
		}
		if (dStyle.right != "auto") {
			ret.right = dStyle.right;
		}
		return ret;
	},
	
	_updatePosition: function(/*Object*/ style, /*int*/ dx, /*int*/ dy) {
		// summary: updates a size object using dx/dy, the style object pass in will be updated
		// style: CSS style object
		// dx: change in x direction in pixels
		// dy: change in y direction in pixels
		var width = parseInt(style.width);
		var height = parseInt(style.height);
		if (width && dx) {
			if (!ibmCfg.themeConfig.isRTL) {
				style.width = width - dx + "px";
			} else {
				style.width = width + dx + "px";
				// adjust the left parameter in RTL mode 
				// so the widget doesn't jump to the right
				var left = parseInt(style.left);
				if (left) {
					style.left = left - dx + "px";
				}
			}
		}
		if (height && dy) {
			style.height = height - dy + "px";
		}
	}
});
