if(!dojo._hasResource["com.ibm.widgets.SelectionModeled"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.widgets.SelectionModeled"] = true;
dojo.provide("com.ibm.widgets.SelectionModeled");

dojo.require("com.ibm.widgets.TreeModeled");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.bundles", "Theme");

dojo.declare( "com.ibm.widgets.SelectionModeled",
			com.ibm.widgets.TreeModeled,
			{
				// summary:
				//		Mixin for building widgets from a selection model, which is a list structure
				//		that represents a current selection in a model starting at the root and walking
				//		down a tree to the currently selected node.
				
				// level: Number the level of the current modeled widget in a heirarchy of modeled
				//		widgets.  This represents the level in the tree from the root where this
				//		widget is rooted.  From the selection model's point of view, it is the index
				//		into the reverse selection path (since that goes up the tree instead).
				level: 0,
				
				// selectionModel: An instance of the com.ibm.data.SelectionModel class representing
				//		a selection model.
				selectionModel: null,
				
				// selectedClass: String name of the CSS class to apply to a node that is in the
				//		selection path.
				selectedClass: "",
				
				// selectedChild: Child widget which is currently marked as selected, indicating that
				//		it is part of the selection model.
				selectedChild: null,
				
				// tiedToSelection: Boolean flag indicating if this widget is tied to the selection
				//		model at this level.  If this is true, then if the selection path changes
				//		at this level, this widget will reload itself completely to root itself
				//		at the new item in the selection path at this level.
				tiedToSelection: false,
				
				constructor: function(){
					this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Theme");
					this.currentlySelected = this.bundle["currently_selected"];
				},
				postMixInProperties: function() {
					this.inherited(arguments);
					this.attr("level", this.getVariedLevel());	// initialize ourselves with level-varying support
				},
				initializeModelWidget: function() {
					this.inherited(arguments);
					this.connect(this.selectionModel, "onSelectionPathChange", "_updateSelection");
				},
				loadHandler: function(items) {
					if(this.selectedChild) {
						delete this.selectedChild;
					}
					return this.inherited(arguments);
				},
				_itemIsEqual: function(item1, item2) {
					return (item1 && this.model.getIdentity(item1)) == (item2 && this.model.getIdentity(item2));
				},
				_setLevelAttr: function(/*String*/ value){
					this.level = value;
					if(this.tiedToSelection) {
						this.item = this.selectionModel.atLevel(this.level, true);
					}
				},
				_markSel: function(childWidget, value) {
					if(value) {
						if(this.selectedChild) {
							this.markSelected(this.selectedChild, false);
						}
						this.selectedChild = childWidget;
					}
					this.markSelected(childWidget, value);
				},
				markSelected: function(childWidget, value) { 
					// summary: Marks the childWidget as selected or unselected depending on the value.
					//		By default, adds/removes the selectedClass to the childWidget's domNode.
					//		Modules that mix this module in can override it to support DOM modifications
					//		that are specific to their markup since this default implementation is
					// 		agnostic to any specific markup.
                    if (childWidget.domNode) {
                        dojo[value ? "addClass" : "removeClass"](childWidget.domNode, this.selectedClass);
                    }
				},
				getVariedLevel: function() {
					// summary: Extension point at which plugins can vary the level at which this
					// 		SelectionModeled widget is tied to.  This is called when the selection path
					// 		changes to determine how the widget should render itself on that change
					// 		event.  If this returns a level that is different than the previous level, 
					// 		the widget will reload itself from scratch and root itself at the item
					//		indexed in the selection path at the level returned by this function.
					//		If this returns the same level as before, no special handling is performed
					//		besides normal widget processing for selections.
					return this.level;
				},
				_updateSelection: function() {
					var oldLevel = this.level, variedLevel = this.getVariedLevel();
					if(oldLevel != variedLevel) {
						// reroot ourselves, and reload
						this.attr("level", variedLevel);
						this.reload(true);
					}
					else {
						var curLevelItem = this.selectionModel.atLevel(this.level, true),
							lastLevelItem = this.selectionModel.atLevel(this.level + 1, true);
						if(curLevelItem != null){
							if(this.tiedToSelection && !this._itemIsEqual(this.item, curLevelItem) ) {
								this.item = curLevelItem;
								this.reload(true);
							}
							else {
								var curLevelId = curLevelItem ? this.model.getIdentity(curLevelItem) : null,
									levelId = lastLevelItem ? this.model.getIdentity(lastLevelItem) : null;
								
								if(curLevelItem && curLevelId != levelId && this.items[curLevelId]) {
									this._markSel(this.items[curLevelId], false);
								}
								if(levelId && this.items[levelId]) {
									this._markSel(this.items[levelId], true);
								}
							}
						}
					}
				},
				_setupNewChild: function(item) {
					var childWidget = this.inherited(arguments);
					if(this._itemIsEqual(this.selectionModel.atLevel(this.level + 1, true), item)) {
						this._markSel(childWidget, true);
					}
					return childWidget;
				},
				getInheritableParameters: function(item) {
					return dojo.mixin(this.inherited(arguments), {
						selectionModel: this.selectionModel,
						selectedClass: this.selectedClass
					});
				},
				onModelItemClick: function(widget, item) {
					// summary: Extension point for getting notifications when users click
					//		a widget representing an item in this model.  By default, this
					//		is propogated to all cascading modeled widgets created by the
					//		model so that all descendants fire the same function.
					//		Implementations should ensure that any onclick event on a child 
					// 		widget representing	modeled data calls results in a call to this 
					// 		function with the appropriate arguments. 
					//		For SelectionModeled widgets, this changes the current selection
					//		to the widget's item that was clicked.
					if(widget.disabled) return false;
					var id = this.model.getIdentity(item);
						thisSel = this.selectionModel.atLevel(this.level, true),
						thisSelId = thisSel ? this.model.getIdentity(thisSel) : null;
					if(thisSelId != id) {
						// didn't click the current selection
						try {
							this.selectionModel.changeSelection(item);
						} catch(e) {/*IE fix*/}
						return true;
					}
					return false;
				}
			}
);

}
