/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.enabler.model.state.NavigationStateProcessor");

//<INTERFACE>
/**
 * Interface representing NavigationStateProcessor.NavigationState is internally Represented as an JSON object.<br/>
 * Paramters are predefined.<br/>
 * <code>
 * {<br/>
 *  sid:{<br/>
 *  &nbsp;&nbsp;value: &lt;sid>,<br/>
 *  &nbsp;&nbsp;params:{}<br/>
 *  },<br/>
 *  pid:{<br/>
 *  &nbsp;&nbsp;value: &lt;pid>,<br/>
 *  &nbsp;&nbsp;params:{}<br/>
 *  },<br/>
 *  pageselection:{<br/>
 *  &nbsp;&nbsp;&lt;spaceid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: &lt;pageid>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&lt;spaceid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: &lt;pageid>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;}<br/>
 *  },<br/>
 *  wparams:{<br/>
 *  &nbsp;&nbsp;&lt;wid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"200">,h:&lt;"300">,st:&lt;"NORMAL">},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;cp:{&lt;page:"3">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&lt;wid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"300">,h:&lt;"400">,st:&lt;"MAX">},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;cp:{&lt;page:"4">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;}<br/>
 *  },<br/>
 *  lcparams:{<br/>
 *  &nbsp;&nbsp;&lt;pid>:{<br/>
 *  &nbsp;&nbsp;templateURL: "&lt;dav:filestore/layout-templates/2ColumnEqual>",{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lcid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"70%">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lcid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"30%">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&lt;pid>:{<br/>
 *  &nbsp;&nbsp;templateURL: "&lt;dav:filestore/layout-templates/3ColumnEqual>",{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lcid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"30%">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lcid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"30%">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lcid>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;rp:{w:&lt;"40%">}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>
 *  &nbsp;&nbsp;}<br/>
 *  },<br/>
 *  sparams:{<br/>
 *  &nbsp;&nbsp;&lt;id>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;global: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: &lt;value>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/> 
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: &lt;value>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/> 
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&lt;scope-id>: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: {<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: &lt;value>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/>  
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;},<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name>:{<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;value: &lt;value>,<br/>
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;params:{lm:&lt;timestamp>}<br/> 
 *  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;}<br/>
 *  &nbsp;&nbsp;}<br/>
 *  }<br/>
 * }<br/>
 * </code>
 * <br/>
 * Navigation State parameters will be written to url or persistence store upon configuration.<br/> 
 * Here's the default configuration:<br>
 * <ul>
 * <li>navstate.persistence.url = pid,cp,sparams</li>
 * <li>persistence.pstore = w,h,st,pid,sid,pageselection</li>
 * </ul>
 * <br/>
 * Following is the list of parameters that are configurable:<br/>
 * <ul>
 * <li>pid: page id </li>
 * <li>sid: space id </li>
 * <li>pageselection: page selections </li>
 * <li>sparams: shareable parameter sets </li>
 * <li>h: widget height </li>
 * <li>w: widget width </li>
 * <li>st: widget window state </li>
 * <li>md: widget mode </li>
 * <li>cp: widget customized state </li>
 * </ul>
 * 
 * Upon page loading or page refresh, navigationstatemodel is initialized with navigation state data from url or </br>
 * persistence. "decode" will be used to load data from url and "preprocess" will be used to load data from persistence.</br>
 * Upon commit on NavigationStateModel updates, navigation state data will be sent to url or sent to persistence.</br>
 * "postprocess" will send the updates to persistence and "encode" will generate the new url fragment based on latest navigation state.</br>
 * 
 *
 * @ibm-spi
 * @ibm-module Base
 */
dojo.declare( "com.ibm.mashups.enabler.model.state.NavigationStateProcessor", null, {
	/**
    * Encodes the specified widget instance id.
    * @param {String} wid widget instance id; different page may have same widget instance id.
    * @param {com.ibm.mashups.enabler.model.state.NavigationStateModel} navigationStateModel
    * @type String
    * @return {String} unique id, representing a widget instance on a page
    */
    encodeWidgetIdentifier: function(wid, navigationStateModel){
    },
   /**
    * Decodes a unique widget instance id.
    * @param {String} wid unique widget instance id.
    * @param {com.ibm.mashups.enabler.model.state.NavigationStateModel} navigationStateModel
    * @type String
    * @return {String} widget instance id
    */
    decodeWidgetIdentifier: function(wid, navigationStateModel){
    },
	/**
     * Decode the url and store the state into a JSON Object. JSON object should be
     * pass into Callback. If no callback is defined, navigation state is returned.<br/>
     * <code>callback:function(state){};</code><br/>
     * @param {String} url url string
     * @param {Function} callback callback function
     * @type Object
     * @return {Object} navigation state object
     */
	decode:function(url,callback){
	},
	/**
     * Encode state object and generate fragment. Fragment should be passed into callback function.<br/>
     * <code>callback(fragment);</code>
     * If no callback is defined, url fragment is returned.<br/>
     * @param {Object} state JSON object representing Mashup state
     * @param {Function} callback callback function which takes the following parameters:
     * &nbsp;&nbsp;<b><code>additionalParams</code> (JSON)</b> - if the <code>additionalParams<code> was passed in to the encode 
     * function, then it must be passed back into the callback function <br>
     * &nbsp;&nbsp;<b><code>responseParams</code> (JSON)</b> - <i>Optional</i> If <code>additionalParams.allowRedirect</code> is true and this is an asynchronous commit, 
     * then the implementation of this function may pass <code>responseParams.doRedirect</code> set to true to refresh the page after the commit is completed <br>
     * @param {Object} oldState JSON object representing Mashup state before it gets updated
     * @param {Object} additionalParams <i>Optional</i> JSON object to control various aspects of encoding or the registered callback
     * @param {com.ibm.mashups.enabler.model.state.NavigationStateModel} navigationStateModel
     * @type String 
     * @return {String} the url fragment that contains navigation state
     */
	encode:function(state,callback,oldState, additionalParams, navigationStateModel){
	},
	/**
     * @deprecated Use generateURL instead.
     * 
     * @param {Object} state 
     * @param {Function} callback 
     * @param {JSON} params
     * @type String
     */
	generateUrl:function(state,callback,params){
	},
	/**
     * Encode state object and generate full url. URL should be absolute URL. URL should be passed into callback function.
     * If no callback, full url is returned.<br/>
     * <code>callback:function(url){};</code><br/>
     * Sample parameter: {nohash:"true"}<br/>
     * if "nohash" is set to true,the returned url will not contain state in hash.<br/>
     * By default Lotus Mashups only supports url that contains navigation state in hash.  <br/>
     * Extensions can be added to support additional parameter by using extension registry.<br/>
     * 
     * @param {Object} state JSON object representing Mashup state
     * @param {Function} callback callback function
     * @param {JSON} params additional parameter in json format
     * @type String
     * @return {String} full url that contains navigation state
     */
	generateURL:function(state,callback,params){
	},
	/**
     * Preprocess could be used to filter the original state. For example, 
     * pulling more state information that's cached in cookie.<br/>
     * The final state should be passed into callback.<br/>
     *   <code>callback:function(state){};</code><br/>
     * If no callback, state object will be returned.<br/>
     * @param {Object} state JSON object representing Mashup state
     * @param {Function} callback callback function
     * @type Object
     * @return {Object} object contains navigation state
     */
	preprocess:function(state,callback){
	},
	/**
     * Dispose the any navigation state that's persisted. Callback is executed when dispose action is done. <br/>
     * <code>callback:function(){};</code>
     * @param {Function} callback callback function
     * @type void
     */
	dispose:function(callback){
	},
	/**
     * Postprocess could be used to filter the original state before url is generated.
     * For example,some state information should be persisted into cookie.<br/>
     * The final state should be passed into callback.<br/>
     *   <code>callback:function(state){};</code><br/>
     * If no callback, state object will be returned.<br/>
     * @param {Object} state JSON object representing Mashup state
     * @param {Function} callback callback function
     * @param {Object} oldState JSON object representing Mashup state before it gets updated
     * @param {Object} additionalParams <i>Optional</i> JSON object to control various aspects of encoding or the registered callback.
     * @type Object
     * @return {Object} object contains navigation state
     */
	postprocess:function(state,callback,oldState, additionalParams){
	}
});

//</INTERFACE>

dojo.require("com.ibm.mm.enabler.model.state.NavigationStateProcessorImpl");
