/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.iwidget.iContext");

dojo.require("com.ibm.mashups.iwidget.Constants");
dojo.require("com.ibm.mashups.iwidget.iEvents");
dojo.require("com.ibm.mashups.iwidget.io");

//<INTERFACE>
/**
 * This interface defines methods that the iWidget can use to interact with the iContext.
 * An iContext instance is created specifically for each iWidget instance and a reference will be set in iWidget encapsulation object.
 * Thus within iWidget encapsulation class, iContext should be referred to by using <b>"this.iContext"</b><p/>
 *  &nbsp;&nbsp;&nbsp; <code>var root = this.iContext.getRootElement();</code><p/>
 *
 * @ibm-api
 */
dojo.declare("com.ibm.mashups.iwidget.iContext", null, {

    /**
     * @private
     */
    constructor: function() {
    },
    /**
     * Defines constants available in iContext.<br/>
     * Constants defined within iContext can be referred as follows:</p>
     *
     *  &nbsp;&nbsp;&nbsp; <code>this.iContext.constants.mode.VIEW</code><br/>
     *  &nbsp;&nbsp;&nbsp; <code>this.iContext.constants.mode.EDIT</code><br/>
     *  &nbsp;&nbsp;&nbsp; <code>this.iContext.constants.mode.PERSONALIZE</code><br/>
     *  &nbsp;&nbsp;&nbsp;<code>this.iContext.constants.mode.HELP</code><br/>
     *  &nbsp;&nbsp;&nbsp;<code>this.iContext.constants.ATTRIBUTES</code><br/>
     *  &nbsp;&nbsp;&nbsp;<code>this.iContext.constants.IDESCRIPTOR</code><br/>
     *  &nbsp;&nbsp;&nbsp;<code>this.iContext.constants.USERPROFILE</code><br/>
     * </code>
     * @type Object
     * @ibm-module iWidget2
     */
    constants: {
        /*@type object*/
        mode: {
            /*@type String*/
            VIEW:/*String*/ "view",//The generated markup fragment(s) should reflect normal interaction with the iWidget
            /*@type String*/
            EDIT:/*String*/ "edit",//The generated markup fragment(s) should reflect iWidget interactions for editing iWidget attributes scoped to this instance of iwidget, but applicable to all users
            /*@type String*/
            PERSONALIZE:/*String*/ "personalize",//The generated markup fragment(s) should reflect iWidget interactions for personalizing iwidget attributes scoped to this instance on the iWidget for this user
            /*@type String*/
            CONFIG:/*String*/ "config",//The generated markup fragment(s) should reflect iWidget interactions for editing iwidget attributes scoped to all instances of the iWidget for all users		   
            /*@type String*/
            HELP:/*String*/ "help"//The iWidget should generate markup fragment(s) to assist the user with interacting with the iWidget
        },
        event: {
            /*@type String*/
            TITLE:/*String*/ "title",//Name for the Attribute holding event title
            /*@type String*/
            DESCRIPTION:/*String*/ "description"//Name for the Attribute holding event description
        },
        /*@type String*/
        ATTRIBUTES:/*String*/ "attributes", //Name for the ManagedItemSet holding the customization attributes.
        /*@type String*/
        IDESCRIPTOR:/*String*/ "idescriptor", //Name for the ManagedItemSet holding the items describing the iWidget
        /*@type String*/
        USERPROFILE:/*String*/ "userprofile", //Name for the ManagedItemSet holding data about the user
        /*@type Object*/
        keys: {
            /*@type int*/
            SHIFT:/*int*/ 1,
            /*@type int*/
            ALT:/*int*/ 2,
            /*@type int*/
            CTRL:/*int*/ 4,
            /*@type int*/
            META:/*int*/ 8,
            /*@type int*/
            CAPSLOCK:/*int*/ 16
        },
        windowstate: {
            /*@type String*/
            NORMAL:/*String*/ "normal",
            /*@type String*/
            MINIMIZE:/*String*/ "minimize",
            /*@type String*/
            MAXIMIZE:/*String*/ "maximize"
        },
        status: {
            /*@type int*/
            SUCCESS:/*int*/ 200,
            /*@type int*/
            TIMEOUT:/*int*/ 408,
            /*@type int*/
            NOTFOUND:/*int*/ 404,
            /*@type int*/
            INTERROR:/*int*/ 500,
            /*@type int*/
            OTHER:/*int*/ 303
        },
        changeType: {
            /*@type String*/
            CHANGEDVALUE:/*String*/ "changedValue",
            /*@type String*/
            NEWITEM:/*String*/ "newItem",
            /*@type String*/
            REMOVEDITEM:/*String*/ "removedItem"
        
        }
    },
	/**
     * This method returns the unique identifier for this instance of the iWidget in the current evironment (usually a web page). 
     * @type String
     * @return{String}  widgetId unique identifier of the iWidget. return <code>null</code> if element is not available.
     * @ibm-module iWidget2
     */
    getWidgetId: function() {
        return null;
    },
    /**
     * Returns the iContext supplied element containing the content for the current mode of the iwidget such that the iWidget can easily do things such as search its own markup.
     * @type DOMElement
     * @return{DOMElement}  rootElement of the iWidget. return <code>null</code> if element is not available.
     * @ibm-module iWidget2
     */
    getRootElement: function() {
        return null;
    },
    /**
     * This method provides the same semantics as the DOM method with the same name, but restricts the search to the iWidget's markup rather than the entire page
     * @param{String} id  element id. Must not be <code>null</code>.
     * @param{DOMElement} root  optional DOMElement to define a search scope, it will search in the scope of current active mode if root is not provided.
     * @type DOMElement
     * @return{DOMElement} return the element, or <code>null</code> if element is not found.
     * @ibm-module iWidget2
     */
    getElementById: function(/*String*/id,/*DOMNode*/ root) {
        return null;
    },
    /**
     * This method returns the ManagedItemSet that provides access to the iWidget's customization attributes.
     * If there is no ManagedItemSet  related to the iWidget's customization attributes, this method MUST create an empty set and return it.
     * @type com.ibm.mashups.iwidget.itemset.ManagedItemSet
     * @return{com.ibm.mashups.iwidget.itemset.ManagedItemSet } return the ManagedItemSet or empty ManagedItemSet if no customization attributes is found.
     * @ibm-module iWidget2
     */
    getiWidgetAttributes: function() {
        return {};
    },
    /**
     * This method returns an ItemSet corresponding to the requested name. If it does not already exist, an ItemSet will be created and associated with the supplied name.
     * @param{String}name name of ItemSet. Must not be <code>null</code>.
     * @param{Boolean}isPrivate deprecated,please use getShareableItemSet()if it's a ShareableItemSet.
     * @type com.ibm.mashups.iwidget.itemset.ItemSet
     * @return{com.ibm.mashups.iwidget.itemset.ItemSet} return the requested ItemSet or <code> null</code> if access is denied.
     * @ibm-module iWidget2
     */
    getItemSet: function(/*String*/name,/*Boolean*/ isPrivate) {
        return null;
    },
	  /**
     * This method returns an ShareableItemSet corresponding to the requested name. If it does not already exist, an ItemSet will be created and associated with the supplied name.
     * @param{String}name name of ItemSet. Must not be <code>null</code>.
     * @type com.ibm.mashups.iwidget.itemset.ItemSet
     * @return{com.ibm.mashups.iwidget.itemset.ShareableItemSet} return the requested ItemSet or <code> null</code> if access is denied.
     * @ibm-module iWidget2
     */
    getShareableItemSet: function(/*String*/name) {
        return null;
    },
    /**
     * Provides means for iWidget to declare dependency on set of shared resource support dynamic loads in asynchronous manner. JS and CSS resources are supported.
     * @param{String} globalid  globalid of the Item that needs to be loaded. Must not be <code>null</code>.
     * @param{String}version  optional 
     * @param{String}uri uri of the resource. Must not be <code>null</code>.
     * @param{Object}cb optional  
     * @param{String}mimeType  optional
     * @type void
     * @ibm-module iWidget2
     */
    requires: function(/*String*/globalid,/*String*/ version,/*String*/ uri,/*function*/ cb,/*String*/ mimeType) {
    },
    /**
     * This method returns an instance of type Object which was initialized prior to the loading of the iWidget (either as a generic Object or an instance of the encapsulation Object ) . Its purpose is to support proper
     * encapsulation of the iWidget's assets (variables and methods) such that multiple instances of the iWidget can be loaded into a
     * single page's DOM without stepping on each other.
     * So iWidget resources can refer to any functions that's defined within iWidget encapsulation object by using iContext.iScope().fn();
     * @type Object
     * @return{Object} return an instance of the encapsulation class or return a generic object
     * @ibm-module iWidget2
     */
    iScope: function() {
        return null;
    },
    /**
     * This method requests the iContext to process the markup such that it can be inserted into the iWidget's markup and properly interact with the page.
     * (i.e. all the iContext reference in the markup needs to be properly namespaced). Where, when and how the markup is inserted into the page
     * is the responsibility of the iWidget.
     * @param{String}markup  markupString that needs to be processed. Must not be <code>null</code>.
     * @type String
     * @return  On success this method MUST return the processed markup while on failure it MUST return null
     * @ibm-module iWidget2
     */
    processMarkup: function(/*String*/markup) {
        return null;
    },
    /**
     * This method requests the iContext to process the subtree under the supplied node for the purpose of resolving and instantiating any referenced iWidgets.
     * @param{DOMNode}root root element of the subtree. Must not be <code>null</code>.
     * @type void
     * @ibm-module iWidget2
     */
    processiWidgets: function(/*DOMNode*/root) {
        //summary: This method requests the iContext to process the subtree under the supplied node for the purpose of resolving and instantiating any referenced iWidgets.
        //root: root element of the subtree
    },
    /**
     * This method returns an array of Elements within the iWidget's markup which have the supplied value as one of those specified by the Element's "class" attribute.
     * @param{String}classname  name of class attribute. Must not be <code>null</code>.
     * @param{DOMElement}root root element of the subtree. Must not be <code>null</code>
     * @type DOMElement[]
     * @return{DOMElement[]} return an array of elements
     * @ibm-module iWidget2
     */
    getElementByClass: function(/*String*/classname,/*DOMElement*/ root) {
        return null;
    },
    /**
     * This method returns the ManagedItemSet that provides access to the user's profile data.
     * @type com.ibm.mashups.iwidget.itemset.ManagedItemSet
     * @return{com.ibm.mashups.iwidget.itemset.ManagedItemSet } if there is no ManagedItemSet related to the user's profile, this method creates an empty set and returns it. If access to the user's profile is denied, this method returns null.
     * @ibm-module iWidget
     */
    getUserProfile: function() {
        return null;
    },
    /**
     * This method returns the ManagedItemSet that provides access to iWidget's descriptive items.
     * @type com.ibm.mashups.iwidget.itemset.ManagedItemSet
     * @return {com.ibm.mashups.iwidget.itemset.ManagedItemSet } If there is no ManagedItemSet  related to the iWidget's descriptive items this method creates an empty set and returns it..
     * @ibm-module iWidget2
ja     */
    getiDescriptor: function() {
        return null;
    },
    /**
     * This object provides access to io service. io service is accessible by using "this.iContext.io".<p/>
     * @type com.ibm.mashups.iwidget.io
     * @ibm-module iWidget2
     */
    io: {},
    /**
     * This object provides access to events service. event service is accessible by using "this.iContext.iEvents".<p/>
     * @type com.ibm.mashups.iwidget.iEvents
     * @ibm-module iWidget
     */
    iEvents: {}
});
//</INTERFACE>
dojo.require("com.ibm.mm.iwidget.icontext.IContextDefaultImpl");
