/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.iwidget.services.EventService");

/**
 * This interface defines iWidget EventService which provides  a topic driven subscribe/publish mechanism. 
 * "fireEvent" and "broadcastEvent"  are provided as convenience functions on top of this basic model.
 * The service can be retrieved using the following code: <br>
 * <code>var eventService = com.ibm.mashups.services.ServiceManager.getService(<br/>
 * &nbsp;&nbsp;&nbsp;&nbsp;com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME);</code><br/>
 * @ibm-spi
 * @ibm-module iWidget2
 */
dojo.declare("com.ibm.mashups.iwidget.services.EventService",null,  {
    /**
     * The service name to be used to fetch the service from the ServiceManager
     * @type String
     */
    SERVICE_NAME: "eventService",
    
  	/**
     * @private
     */
    constructor:function(){
    },

/**
Allows page component to invoke a handled event that's defined in an iWidget.<br/>
For example,  in Lotus Mashups, user can click "save" to save the attributes when user finished editing a widget .<br/>
Then the component that renders the "save" button could use this api to distribute an onModeChanged event to the widget.<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>com.ibm.mashups.services.ServiceManager.getService("eventService").fireEvent(this.iwidgetId,"onModeChanged", {newMode:"view"});	 </code><br/>
So the widget could get notified and go back to "view"mode.<br/>
Same function can also be achieved by using "publishEvent"<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>com.ibm.mashups.services.ServiceManager.getService("eventService").publishEvent("widgetevents."+this.iwidgetId+"."+"onModeChanged", {newMode:"view"}); </code><br/>
@param{String } targetWidget  id of target iWidget. Must never be NULL.
@param{String } targetEvent  event name of target Event .Must never be NULL.
@param{Object } payload  optional. data object that's distributed with an event. May be NULL.
@param{String } payloadType  optional. type of the payload
@param{String } sourceid  optional. id of source component that triggers this event
@type void
*/
      fireEvent: function(targetWidget,targetEvent,payload,payloadType,sourceid){
      },	
/**
Allows page components to broadcast  an event to all the widgets/components on the page. There's no need for widgets to subscribe that topic first.<br/>
For example,  mode selector iWidget may broadcast a "pageModeChanged" event when page mode is changed from view mode to edit mode.<br/>
Then all the widgets/components  will get notified and all widgets/components that  can handle "pageModeChanged" event will update accordingly.<br/>
A widget that can handle this event must have this event defined. <br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code> &lt;iw:event id="pageModechanged"  eventDescName="pageModeChanged_desc" handled="true"  onEvent="handlePageModeChange" /&gt; </code><br/>
A component that can handle this event must subscribe this event by using:<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <code>serviceManager.getService("eventService").subscribeEvent ("pageModechanged",handlerFn,scope,sourceid)</code><br/>
Code example:<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>com.ibm.mashups.services.ServiceManager.getService("eventService").broadcastEvent("pageModeChanged", payload,payloadtype,,sourceid); </code><br/>
Same function can also be achieved by using "publishEvent"<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<code>com.ibm.mashups.services.ServiceManager.getService("eventService").publishEvent("*"+"."+"onModeChanged", {newMode:"view"}); </code><br/>
@param{String } targetEvent  event name of target Event .Must never be NULL.
@param{Object } payload  optional.  data object that's distributed with an event. May be NULL.
@param{String } payloadType  optional.  type of the payload
@param{String } sourceid  optional. id of client component that triggers this event.
@type void
*/ 	 	  
      broadcastEvent: function(targetEvent,payload,payloadType,sourceid){
      },
/**
Allows page components to switch to a different page and broadcast an event to all the widgets/components on that page.<br/>
@param{String } targetEvent  event name of target Event .Must never be NULL.
@param{Object } payload  optional.  data object that's distributed with an event. May be NULL.
@param{String } payloadType  optional.  type of the payload
@param{String } sourceid  optional. id of client component that triggers this event.
@param{String } pageid  optional. id of page to switch to and receive the event. If none is provided, no page switch will occur and event will broadcast to current page.
@type void
*/ 	 	  
      broadcastEvent: function(targetEvent,payload,payloadType,sourceid,pageid){ // JSLINT-IGNORE: cannot change public API
      },
/**
Allows page components to switch to a different page and space and broadcast an event to all the widgets/components on that page.<br/>
@param{String } targetEvent  event name of target Event .Must never be NULL.
@param{Object } payload  optional.  data object that's distributed with an event. May be NULL.
@param{String } payloadType  optional.  type of the payload
@param{String } sourceid  optional. id of client component that triggers this event.
@param{String } pageid  optional. id of page to switch to and receive the event. If none is provided, no page switch will occur and event will broadcast to current page.
@param{String } spaceid  optional. id of space containing the page to switch to and receive the event. If none is provided, current space is assumed.
@type void
*/ 	 	  
      broadcastEvent: function(targetEvent,payload,payloadType,sourceid,pageid,spaceid){ // JSLINT-IGNORE: cannot change public API
      },
/**
Allows page components to broadcast more than one event to all the widgets/components on the page.<br/>
@param{Array } eventsArray   Array of objects each containing the following event information: &#123; "targetEvent": targetEvent, "payload": payload, "payloadType": payloadType &#125;. Must never be NULL.
@param{String } sourceid  optional. id of client component that triggers this event.
@type void
*/ 	 	  
      broadcastEvents: function(eventsArray,sourceid){
      },
/**
Allows page components to switch to a different page and broadcast more than one event to all the widgets/components on that page.<br/>
@param{Array } eventsArray   Array of objects each containing the following event information: &#123; "targetEvent": targetEvent, "payload": payload, "payloadType": payloadType &#125;. Must never be NULL.
@param{String } sourceid  optional. id of client component that triggers this event.
@param{String } pageid  optional. id of page to switch to and receive the events. If none is provided, no page switch will occur and events will broadcast to current page.
@type void
*/ 	 	  
      broadcastEvents: function(eventsArray,sourceid,pageid){ // JSLINT-IGNORE: cannot change public API
      },
/**
Allows page components to switch to a different page and space and broadcast more than one event to all the widgets/components on that page.<br/>
@param{Array } eventsArray   Array of objects each containing the following event information: &#123; "targetEvent": targetEvent, "payload": payload, "payloadType": payloadType &#125;. Must never be NULL.
@param{String } sourceid  optional. id of client component that triggers this event.
@param{String } pageid  optional. id of page to switch to and receive the events. If none is provided, no page switch will occur and events will broadcast to current page.
@param{String } spaceid  optional. id of space containing the page to switch to and receive the events. If none is provided, current space is assumed.
@type void
*/ 	 	  
      broadcastEvents: function(eventsArray,sourceid,pageid,spaceid){ // JSLINT-IGNORE: cannot change public API
      },
/**
Allows page component to publish a global event that's available to all the other page components.<br/>
Event service supports global topics. For these topics all the page components could subscribe to it.<br/>
For example. when an iWidget is removed from page, a global event -"/enabler/unloadWidget" will be published by the skin component.<br/>
iWidget container is a subscriber of this topic, thus it can clean up the resource related to this iWidget accordingly.<br/>
@param{String } topic  topic that's published.Must never be NULL.
@param{Object } payload  optional. payload object.
@param{String } payloadType optional.  type of the payload
@param{String } sourceid  optional. id of client component that triggers this event.
@type void
*/ 	  
      publishEvent: function(topic,payload,payloadType,sourceid){
      },
/**
Allows page component to subscribe  a global event that's available to all the other page components.<br/>
Event service supports global topic. for these topics all the page listeners could subscribe to it.<br/>
For example. when an iWidget is removed from page, a global event -"/enabler/unloadWidget" will be published by the skin component.<br/>
iWidget container is a subscriber of this topic, thus it can clean up the resource related to this iWidget accordingly.
@param{String } event  event name 
@param{Object } object  optional. scope object of the handlerFn. default scope is global scope.
@param{String} eventCallback optional. eventCallback that will be invoked when event is published
@param{String} subscribeCallback optional. callback  to tell is subscription is good
@param{String } sourceid  optional. id of client component that subscribes this event
@type void
*/	  
      subscribeEvent:function(event,object,eventCallback,subscribeCallback,sourceid){
      },
/**
Allows page component to unsubscribe  a global event that's available to all the other page components.<br/>
@param{Object } subscriptionHandler   subscriptionHandler
@param{String } sourceid  optional. id of client component that subscribes this event
@type void
*/
	  unsubscribeEvent:function(subscriptionHandler,sourceid){
	  }
});
// make sure we can reference this globally
com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME = "eventService";
