/* Copyright IBM Corp. 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer");

/**
 * JavaScript for dataViewer
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.number");
dojo.require("dojox.color");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.Store");
dojo.require("com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil");

(function() {

var dvUtil = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerUtil;

dojo.declare("com.ibm.mm.widget.dataViewer._WB_VER_.v210.widget.DataViewer", [dijit._Widget,dijit._Templated, dijit._Container], {
    Id: "",
    
    data: null,
	
	dataViewerStyle: null,
	
	columnsOrder: [],
    
    rowsPerPage: 5,
    
    currentPage: 0,
    
    pageLinkMax: 0,
    
    pageLinkMin: 0,
    
    typeList: null,
    
    columnList: null,
    
    columnFilter: null,
    
    comVariable: null,
	
    imageBase: "/dataViewer/images/",	// Will be rewritten with current context root
    
    SORTTYPELIST: ["String","Number"],
    
	valueField: "Id",
	
	store: null,
	MAXSORTABLE: 1,  // how many columns can be sorted at once.
	columns: [],
	sortInformation: [{index:0,direction:0}],
	
	headerClass: "notSelected",
	headerUpClass: "sortedUp",
	headerDownClass: "sortedDown",
	rowSelectedClass: "selected",
	rowHighlightClass: "highlighted",

	allowCellSelection: true,
	selectedColumnIndex: -1,

	templateString: '<div></div>',	

    postMixInProperties: function() {
		this.dataViewerStyle = this.dataViewerStyle || {
            titleValue: null,
            titleFont: null,
            titleAlign: null,
            titleFontBold: null,
            titleFontItalic: null,
            titleFontColor: null,
            titleFontSize: null,
            showHeader: null,
            headerFont: null,
            headerFontSize: null,
            headerFontBold: null,
            headerFontItalic: null,
            headerFontColor: null,
            tableFont: null,
            tableFontSize: null,
            tableFontBold: null,
            tableFontItalic: null,
            tableFontColor: null,
            columnLines: null,
            rowLines: null,
            oddRowColor: "#FFFFFF",
            evenRowColor: "#F9FBFC",
            highlightWithColor: "#375E8A",
            columnsAlign : {}
        };		
	},

	postCreate: function(){
	},
	
	uninitialize: function(){
		this.clearRow();
	},
	
    setID: function(id){
        this.Id = id;
    },
    
    getID: function(){
        return this.Id;
    },
    
    setRowsPerPage: function(rows){
        this.rowsPerPage = rows;
    },
    
    fetchFirstPage: function(){
    	this.currentPage = 0;
        this.renderRow(0, parseInt(this.rowsPerPage, 10));
    },
    
    fetchNextPage: function(interval){
        var ivl = interval || 1;
        if ((this.currentPage + 1) * this.rowsPerPage >= this.data.length || this.data.length === 0) {
            return; //till the last page
        } else {
        	this.currentPage = this.currentPage + ivl;
            this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
        }
    },
    
    fetchPage: function(index){
    	this.currentPage = index;
        this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
    },
    
    fetchPrePage: function(interval){
        var ivl = interval || 1;
        if (this.currentPage === 0 || this.data.length === 0) {
            return; //till the last page
        }
        else {
        	this.currentPage = this.currentPage - ivl;
            this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
        }
    },
    
    fetchNextPageRegion: function(){
        var tpage = this.calculatePageCount();
        this.pageLinkMax = (this.pageLinkMax + 5 < tpage) ? (this.pageLinkMax + 5) : (tpage - 1);
        
    	this.currentPage = this.pageLinkMax - 2;
        this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
        
        this.pageLinkMin = this.pageLinkMax - 4;
    },
    
    fetchPrePageRegion: function(){
        var tpage = this.calculatePageCount();
        this.pageLinkMin = (this.pageLinkMin - 5 > 0) ? (this.pageLinkMin - 5) : (0);

    	this.currentPage = this.pageLinkMin + 2;
        this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
        
        this.pageLinkMax = this.pageLinkMax + 4;
    },
    
    getRowsCount: function(){
        return this.store.get(true).length;
    },
    
    calculatePageCount: function(){
        var totalRows = this.getRowsCount();
        if (totalRows === 0) 
            return 0;
        var divide = Math.floor(totalRows / this.rowsPerPage)
		return (this.rowsPerPage * divide === totalRows) ? divide : divide + 1
    },
    
    //TODO, createPageLink needs refactoring
    createPageLink: function(attributes){
		var resourceBundle = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages;
		//recognize pipeline data
		//comment following codes to solve 17925 defect
//		if (this.paginationData.first || this.paginationData.last || 
//			this.paginationData.previous || this.paginationData.next ){
//	        var link = "";		// previous | next
//			var pageIndex = "";	// Page: 1 | 2 | 3 | ...
//			var tpage = Math.floor(this.paginationData.totalResult / this.rowsPerPage);
//			if (tpage * this.rowsPerPage < this.paginationData.totalResult) tpage++;
//			var currentPage = this.paginationData.startIndex / this.rowsPerPage;
//
//	        if (currentPage !== 0) 
//	            link += "<a role='button' title='" + resourceBundle.I_TABLE_PREVIOUS_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().getServerDataPage(" + (currentPage - 1) + ")'>" + (resourceBundle ? resourceBundle.L_TABLE_PREVIOUS : "Previous") + "</a> ";
//			else 
//				link += (resourceBundle ? resourceBundle.L_TABLE_PREVIOUS : "Previous");
//			
//			if (currentPage < 3) {
//				for (var i = 0; i < 5 && i < tpage; i++) {
//					if (i > 0) {
//						pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
//					}
//					if (i !== currentPage) 
//						pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getServerDataPage(" + i + ")'>" + (i + 1) + "</a>";
//					else 
//						pageIndex += (i + 1);
//				}
//				this.pageLinkMin = 0;
//				this.pageLinkMax = i - 1;
//			}
//			else 
//				if (this.pageLinkMax === (tpage - 1) && (this.pageLinkMax - currentPage) < 3) {
//					for (var i = this.pageLinkMin; i <= this.pageLinkMax && i < tpage; i++) {
//						if (i > this.pageLinkMin) {
//							pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
//						}
//						if (i !== currentPage) 
//							pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getServerDataPage(" + i + ")'>" + (i + 1) + "</a>";
//						else 
//							pageIndex += (i + 1);
//					}
//				}
//				else {
//					for (var i = currentPage - 2; i < currentPage + 3 && i < tpage; i++) {
//						if (i > currentPage - 2) {
//							pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
//						}
//						if (i !== currentPage) 
//							pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getServerDataPage(" + i + ")'>" + (i + 1) + "</a>";
//						else 
//							pageIndex += (i + 1);
//					}
//					
//					this.pageLinkMin = currentPage - 2;
//					this.pageLinkMax = i - 1;
//				}
//
//	        if (currentPage !== (tpage - 1)) {
//	            link += " <span class='pageLinkSeparator'>|</span> <a role='button' title='" + resourceBundle.I_TABLE_NEXT_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().getServerDataPage(" + (currentPage + 1) + ")'>" + (resourceBundle ? resourceBundle.L_TABLE_NEXT : "Next") + "</a>";			
//	        } else {
//				link += " <span class='pageLinkSeparator'>|</span> " + (resourceBundle ? resourceBundle.L_TABLE_NEXT : "Next");
//			}
//
//
//			var bottom_link = "<div class='pageIndex'>" + (resourceBundle ? resourceBundle.L_TABLE_PAGE : "Page") + "&nbsp;" + pageIndex + "</div>" + 
//							"<div class='pageBtn'>" + link + "</div>";
//	        return bottom_link;
//
//		}
//comment above codes to solve 17925 defect

        var links = [];
        var link = "";		// previous | next
		var pageIndex = "";	// Page: 1 | 2 | 3 | ...
        var tpage = this.calculatePageCount();
        if (tpage === 0) 
            return "";

		// PMR 20476 - If user doesn't want navigation links displayed when total pages = 1
		var allowPageNavigationHiding = attributes.getItemValue("allowPageNavigationHiding")
		if (allowPageNavigationHiding === "true") {
			if (tpage === 1)  {
				return "";
			}
		}
            
        if (this.currentPage !== 0) 
            link += "<a role='button' title='" + resourceBundle.I_TABLE_PREVIOUS_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().pre()'>" + (resourceBundle ? resourceBundle.L_TABLE_PREVIOUS : "Previous") + "</a> ";
		else 
			link += "<span class='pageBtnDisabled'>" + (resourceBundle ? resourceBundle.L_TABLE_PREVIOUS : "Previous") + "</span>";					
        
		if (this.currentPage < 3) {
			for (var i = 0; i < 5 && i < tpage; i++) {
				if (i > 0) {
					pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
				}
				if (i !== this.currentPage) 
					pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getPage(" + i + ")'>" + (i + 1) + "</a>";
				else 
					pageIndex += "<b>" + (i + 1) + "</b>";
			}
			
			this.pageLinkMin = 0;
			this.pageLinkMax = i - 1;
		}
		else 
			if (this.pageLinkMax === (tpage - 1) && (this.pageLinkMax - this.currentPage) < 3) {
				if (tpage > 5) 
					pageIndex += " <a role='button' title='" + resourceBundle.I_TABLE_PREVIOUS_REGION_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().getPrePageRegion()'>...</a>&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
				for (var i = this.pageLinkMin; i <= this.pageLinkMax && i < tpage; i++) {
					if (i > this.pageLinkMin) {
						pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
					}
					if (i !== this.currentPage) 
						pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getPage(" + i + ")'>" + (i + 1) + "</a>";
					else 
						pageIndex += "<b>" + (i + 1) + "</b>";
				}
			}
			else {
				pageIndex += " <a role='button' title='" + resourceBundle.I_TABLE_PREVIOUS_REGION_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().getPrePageRegion()'>...</a>&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
				for (var i = this.currentPage - 2; i < this.currentPage + 3 && i < tpage; i++) {
					if (i > this.currentPage - 2) {
						pageIndex += "&nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;";
					}
					if (i !== this.currentPage) 
						pageIndex += " <a role='button' title='" + dojo.string.substitute(resourceBundle.I_TABLE_PAGE_LINK_TITLE, [(i + 1)]) + "' href='javascript:" + this.Id + "iContext.iScope().getPage(" + i + ")'>" + (i + 1) + "</a>";
					else 
						pageIndex += "<b>" + (i + 1) + "</b>";
				}
				
				this.pageLinkMin = this.currentPage - 2;
				this.pageLinkMax = i - 1;
			}
		

        if (this.pageLinkMax !== (tpage - 1)) {
            pageIndex += " &nbsp;<span class='pageLinkSeparator'>|</span>&nbsp;<a role='button' title='" + resourceBundle.I_TABLE_NEXT_REGION_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().getNextPageRegion()'>...</a>";
        }
        if (this.currentPage !== (tpage - 1)) {
            link += " <span class='pageLinkSeparator'>|</span> <a role='button' title='" + resourceBundle.I_TABLE_NEXT_LINK_TITLE + "' href='javascript:" + this.Id + "iContext.iScope().next()'>" + (resourceBundle ? resourceBundle.L_TABLE_NEXT : "Next") + "</a>";			
        } else {
			link += " <span class='pageLinkSeparator'>|</span><span class='pageBtnDisabled'> " + (resourceBundle ? resourceBundle.L_TABLE_NEXT : "Next") + "</span>";			
		}
		var bottom_link = "<div class='pageIndex'>" + (resourceBundle ? resourceBundle.L_TABLE_PAGE : "Page") + "&nbsp;" + pageIndex + "</div>" + 
						"<div class='pageBtn'>" + link + "</div>";
        return bottom_link;
    },
    
	isRowSelected: function(/* HTMLTableRow */row, index){
		//	summary
		//	Returns if the passed row is selected.
		var data = this.store.get();
		return data[index].isSelected;
	},
    
    createRow: function(/* object */obj, index){
        //	summary
        //	Create an HTML row based on the passed object
        var row = dojo.create("tr", {
			value: index
		});

        var ds = this.dataViewerStyle,
		    hc = ds.highlightWithColor,
			ihc = this._getInvertHighContrastColor(hc),
			ish = this.isRowHighlighted(row, index),
			isS = this.isRowSelected(row, index),
        	self = this;
		
		(function (scope, row, rIndex, isHighlighted){
			self._rowConnects.push(dojo.connect(row, "onmouseover", null, function(){
				dojo.addClass(this,"hover");
			},true));
			self._rowConnects.push(dojo.connect(row, "onmouseout", null, function(){
				dojo.removeClass(this,"hover");
			},true));
		})(this, row, index, ish);
        
		dojo.toggleClass(row, this.rowSelectedClass, this.isRowSelected(row, index));
        dojo.toggleClass(row, this.rowHighlightClass, !!ish);
		
		dojo.style(row, {
			backgroundColor: ish ? hc : ((index % 2 === 1) ? ds.evenRowColor : ds.oddRowColor),
			color: ish || isS ? ihc : ds.tableFontColor
		});
		
		dojo.forEach(this.columns, function(col, j) {
			var cell = dojo.create("td", {
				innerHTML: ["<span tabindex='-1'>", (obj.src[col.getField()] || ""), "</span>"].join("")
			}, row);
			dojo.style(cell, {
				borderLeftWidth: ds.columnLines,
				borderRightWidth: ds.columnLines,
				textAlign: ds.columnsAlign[col.field] || "left",
				borderTopWidth: ds.rowLines,
				borderBottomWidth: ds.rowLines
			})
		});
        return row; //	HTMLTableRow
    },
    
    clearRow: function(){
    	if (this._rowConnects){
    		dojo.forEach(this._rowConnects, dojo.disconnect);
    	}
		this._rowConnects = [];
    	
		if (this.tableNode && this.tableNode.tBodies && this.tableNode.tBodies[0]) {
			dojo.empty(this.tableNode.tBodies[0]);
		}
    },
    
    renderRow: function(start, length){
        var data = this.store.get();
		var ds = this.dataViewerStyle;

        this.clearRow();

        var body = this.tableNode.tBodies[0];
		dojo.style(body, {
			fontFamily: ds.tableFont,
			fontSize: ds.tableFontSize,
			fontStyle: (ds.tableFontItalic) ? "italic" : "",
			fontWeight: (ds.tableFontBold) ? "bold" : ""
		});
		
        try{
			for (var i = start; i < start + length && i < data.length; i++) {
	            var row = this.createRow(data[i], i);
	            body.appendChild(row);
	        }
			
			// Initially set the first cell of the table focusable
			dojo.query("tbody tr td span", this.tableNode).at(0).attr("tabindex", 0);
        } catch(e){
			console.error(e);
		}
    },
    
    onSort: function(/* HTMLEvent */e){
        //	summary
        //	Sort the table based on the column selected.
        var source = e.target;
        var row = dvUtil.getParentByType(source, "tr");
        var headers = row.getElementsByTagName("th");        
        var header = dvUtil.getParentByType(source, "th");

        for (var i = 0, length = headers.length; i < length; i++) {
        	var sortimg = dojo.byId(this.Id + "pointImg" + i);
        	sortimg.style.display = "none";
            headers[i].className = "";
            dojo.style(headers[i], "color", this.dataViewerStyle.headerFontColor);
            
            if (headers[i] === header) {
				dojo.style(headers[i], "color", "white");
                if (this.sortInformation.length === 0 || this.sortInformation.length > 0 && this.sortInformation[0].index !== i) {
                    this.sortInformation.unshift({
                        index: i,
                        direction: 0,               // 0: down; 1:up; 2: original order
                        sorttype: this.columns[i]["sortType"]
                    });
                } else {                              // Click on the lastest sorted column
                    this.sortInformation[0] = {
                        index: i,
                        direction: (++this.sortInformation[0].direction) % 3,
                        sorttype: this.columns[i]["sortType"]
                    };
					if (this.sortInformation[0].direction === 2) {
                        dojo.style(headers[i], "color", this.dataViewerStyle.headerFontColor);
					}
                }
            }
        }
		
        this.sortInformation.length = Math.min(this.sortInformation.length, this.MAXSORTABLE);
		
        for (var i = 0; i < this.sortInformation.length; i++) {
            var idx = this.sortInformation[i].index;
            var sortimg = dojo.byId(this.Id + "pointImg" + idx);
			if (this.sortInformation[i].direction !== 2) {
				var dir = (~ this.sortInformation[i].direction) & 1;
				headers[idx].className = dir === 0 ? this.headerDownClass : this.headerUpClass;
				
				sortimg.setAttribute("src", dir === 0 ? this.imageBase + "columnSort_down.gif" : this.imageBase + "columnSort_up.gif");
                dojo.style(sortimg, {
					width: "5px",
					height: "7px",
					display: ""
				});
			} else {
				headers[idx].className = "";
				sortimg.style.display = "none";
			}
        }
        
        this.sort();
    },

	createSorter: function(/* array */info){
		//	summary
		//	creates a custom function to be used for sorting.
		var self=this;
		var sortFunctions=[];	//	our function stack.
	    
		function createSortFunction(fieldIndex, dir, sorttype, sorttypeList){
			if (dir !== 0 && dir !== 1) {
				return function(rowA, rowB) {
					return false;
				};
			}
			dir = (dir === 0) ? 1 : -1;
			
			var meta=self.columns[fieldIndex];
			var field=meta.getField();
			
			if(sorttype && (sorttype === sorttypeList[1])){
				//Sorttype is Number
				return function(rowA, rowB){
					var la = dojo.number.parse(rowA[field]);
					var lb = dojo.number.parse(rowB[field]);
					
					var a = isNaN(la)? parseFloat(rowA[field]) : la;
					var b = isNaN(lb)? parseFloat(rowB[field]) : lb;
					
					// If one of them is NaN, use String type to sort
                    if (isNaN(a) || isNaN(b)) {
                        a = rowA[field];
                        b = rowB[field];
                    }
                    var ret = 0;
					if(a > b) ret = 1;
					if(a < b) ret = -1;
					return dir * ret;
				};
			}else{
				//default is String
				return function(rowA, rowB){
					var a = rowA[field];
					var b = rowB[field];
					var ret = 0;
					if(a > b) ret = 1;
					if(a < b) ret = -1;
					return dir * ret;
				};
			}
		}
        
		var max = Math.min(info.length, this.MAXSORTABLE, this.columns.length);
		for (current = 0; current < max; current++) {
            if (info[current].direction === 0 || info[current].direction === 1) {
                sortFunctions.push(
                    createSortFunction(info[current].index, info[current].direction, info[current].sorttype, this.SORTTYPELIST)
                );
            }
		}
		
		if (sortFunctions.length === 0) 
		    return null;
        
		return function(rowA1, rowB1){
			var rowA = rowA1.src;
			var rowB = rowB1.src;
			var idx=0;
			while(idx < sortFunctions.length){
				var ret = sortFunctions[idx++](rowA, rowB);
				if(ret !== 0) return ret;
			}
			//	if we got here then we must be equal.
			return 0;
		};	//	function
	},
    
	sort: function(){
        var sortFunction = this.createSorter(this.sortInformation);
        if (sortFunction) {
            this.store.get(false).sort(sortFunction);
            this.renderRow(this.currentPage * this.rowsPerPage, parseInt(this.rowsPerPage, 10));
        } else {
			// restore to original order
			this.store.resetClone();
			this.renderFirstPage();
		}
	},
	
    renderTable: function(){
        //	summary
        //	render table title, summary, header. Do not render table body
		
        //For a11y
		var resourceBundle = com.ibm.mm.widget.dataViewer._WB_VER_.v210.dataViewerMessages;
		dojo.attr(this.tableNode, "summary", this.dataViewerStyle.titleValue ? this.dataViewerStyle.titleValue : resourceBundle.I_TABLE_DEFAULT_SUMMARY);

        //Create table title with caption for a11y
		var caption = this.tableNode.caption;
		if (!caption) {
	        caption = dojo.create("caption", {
				innerHTML: this.dataViewerStyle.titleValue
			});
		}
		
		dojo.style(caption, {
			"fontFamily": this.dataViewerStyle.titleFont,
			"textAlign": this.dataViewerStyle.titleAlign,
			"color": this.dataViewerStyle.titleFontColor,
			"fontSize": this.dataViewerStyle.titleFontSize
		});
        if (this.dataViewerStyle.titleFontBold && this.dataViewerStyle.titleFontBold){
        	caption.style.fontWeight = 'bold';
        }
        if (this.dataViewerStyle.titleFontItalic && this.dataViewerStyle.titleFontItalic){
        	caption.style.fontStyle = 'italic';
        }
		
		dojo.place(caption, this.tableNode, "first");
        
        //	if there is no thead, create it now.
        var head = this.tableNode.getElementsByTagName("thead")[0];
		if (!this.dataViewerStyle.showHeader) head.style.display = "none";
        if (head.getElementsByTagName("tr").length === 0) {
            var row = dojo.create("tr");
            for (var i = 0, colen = this.columns.length; i < colen; i++) {
				var focusable = i === 0;
            	var cell = dojo.create("th", {
					id: this.mmWidgetId + "_" + i,
					align: "center",
					scope: "col",
					innerHTML: "<span tabindex='" + (focusable ? "0" : "-1") + "'>" + 
								this.columns[i].label + " " + 
								"<img id='" + this.Id + "pointImg" + i + "' alt='' style='display:none'>" +
								"</span>"
				});
				
				dojo.style(cell, {
					fontStyle: (this.dataViewerStyle.headerFontItalic) ? "italic" : "",
					fontWeight: (this.dataViewerStyle.headerFontBold) ? "bold" : "",
					color: this.dataViewerStyle.headerFontColor,
					fontFamily: this.dataViewerStyle.headerFont,
					fontSize: this.dataViewerStyle.headerFontSize
				});
				
    			//	attach the events.
                if (!this.columns[i].noSort) {
                	this.connect(cell, "onclick", "onSort");
                	this.connect(cell, "onkeypress", "onHeaderCellPress");
                }

                row.appendChild(cell);
            }

            head.appendChild(row);
        }
		
        //No sorting column at default
        this.sortInformation = [];

        this.store.forEach(function(element){
			element.isSelected = false;
		});

        this.domNode.appendChild(this.tableNode);
	},
    
    setStore: function(store){
		this.store = store;
		this.data = [];
		dojo.forEach(this.store.get(true), dojo.hitch(this,function(d){
			this.data.push(d.src);
		}));
	},
	
	renderFirstPage: function() {
		this.renderTable();
		this.fetchFirstPage();
	},
	
    isRowHighlighted: function(/* HTMLTableRow */row, index){
        //	summary
        //	Returns if the passed row is highlighted.
    	var data = this.store.get();
    	var v = data[index];
		return !!(v && v.isHighlighted);
    },
    
	//	parsing functions, from HTML to metadata/SimpleStore
	_meta:{
		field:null,
		format:null,
		filterer:null,
		noSort:false,
		sortType:"String",
		dataType:String,
		sortFunction:null,
		filterFunction:null,
		label:null,
		align:"left",
		getField:function(){ 
			return this.field || this.label;
		},
		getType:function(){ 
			return this.dataType; 
		}
	},
	createMetaData: function(/* object */obj){
		//	summary
		//	Take a JSON-type structure and make it into a ducktyped metadata object.
		for(var p in this._meta){
			//	rudimentary mixin
			if(!obj[p]){
				obj[p] = this._meta[p];
			}
		}
		if(!obj.label){
			obj.label=obj.field;
		}
		return obj;	//	object
	},

    parseColumns: function(data, lables, sortType){
        this.columnList = [];
        this.columns = [];
        for (var i = 0; i < data.columnList.length; i++) {
        
            var columnListItem = {};
            columnListItem["label"] = data.columnList[i];//columnLabel;
            
            columnListItem.filterFunction = null;
            this.columnList.push(columnListItem);
            
            var column = {};
            if (this.columnFilter[i]) {
                column["field"] = data.columnList[i];//columnStringList[i];
                if (i < this.typeList.length) {
                    column["dataType"] = this.typeList[i];
                    if (this.typeList[i] === "Number") 
                        column["align"] = "right";
                }
                column["label"] = lables && lables[i] ? lables[i] : data.columnList[i];
                column["sortType"] = sortType && sortType[i] ? sortType[i] : "String";
                this.columns.push(this.createMetaData(column));
            } else {
				this.columns.push(false);
			}
        }
		if (this.columnsOrder) {
			var orderedColumns = new Array(this.columns.length);
			for (var j = 0; j < this.columns.length; j++) {
				orderedColumns[this.columnsOrder[j]] = this.columns[j];
			}
			this.columns = orderedColumns;
		}
		var finalColumns = [];
		for (var i = 0; i < this.columns.length; i++) {
			if (this.columns[i] !== false) {
				finalColumns.push(this.columns[i]);
			}
		}
		this.columns = finalColumns;

    },
    
	onLoad : function(e) {
		
		this.tableNode = dojo.create('TABLE');
		if(this.tableNode){
			dojo.addClass(this.tableNode, "mumDataViewer-_WB_VER_-v210");

			if(!this.tableNode.getElementsByTagName("thead")[0]){
				this.tableNode.appendChild(dojo.create("thead"));
			}
			
			// if the table doesn't have a tbody already, add one and grab a reference to it
			if (!this.tableNode.tBodies || this.tableNode.tBodies.length < 1) {
				var body = dojo.create("tbody");
				this.tableNode.appendChild(body);
			} else {
				var body = this.tableNode.tBodies[0];
			}
			
			this.connect(body, "onclick", "onSelect");
			this.connect(body, "onkeypress", "onRowCellPress");
		}
		
	},

	//add lablels as input parameter
	fetchAllInfo: function(data, filter, lables, sorttype){

		if (data.result.length > 0 && data.columnList.length > 0) {
	        this.onLoad();

	        this.paginationData = data;
	        
	        this.columnFilter = filter;
            
            this.typeList = []
			for (var i=0; i<data.columnList.length; i++){
				this.typeList.push('text');
			}

            this.parseColumns(data, lables, sorttype);
            
        }
    },

    getDataByRow: function(/*HTMLTableRow*/row){
		var index = parseInt(dojo.attr(row, "value"), 10);
		var data = this.store.get();
		return data[index].src;
	},

	onDataSelected: function(/*object*/data, /*number*/ columnIndex){
        var rowDataColumn = "",
            rowDataType = "",
            rowDataValue = "",
            rowData = "",
            cellValue = "";
        
        if (this.columns.length <= 0) 
            return;

		var firstColumn = true;
        for (var i = 0; i < this.columnList.length; i++) {
			if (!(this.columnFilter[i])) {continue;}
            var column = this.columnList[i];            
            var type = this.typeList[i];
            var value = data[column.label];

            if (i > 0 && !firstColumn) {
                rowDataColumn += ",";
                rowDataType += ",";
                rowDataValue += ",";
            } else {firstColumn = false;}
            rowDataColumn += column.label;
            rowDataType += type;            
            rowDataValue += value;
        }
        rowData = rowDataColumn + "|" + rowDataType + "|" + rowDataValue;
        
		var celljsonData = {};
		var cellKey = this.columns[columnIndex].field;
		celljsonData[cellKey] = data[cellKey];
				
		cellValue = data[cellKey];
		
        dojo.publish("dataViewer/" + this.mmWidgetId + "/dataSelected", [{
            dataInfo: {
                widgetId: this.mmWidgetId,
                rowData: rowData,
                cellValue: cellValue,
                rowJSonData: data,
				cellJSonData: celljsonData
            }
        }]);
    },
    
	toggleSelectionByRow: function(/*HTMLTableRow*/row){
		//	summary
		//	Flips the selection state of object represented by row.

		var data = this.store.get();
		var rowIndex = parseInt(dojo.attr(row, "value"), 10);
		data[rowIndex]['isSelected'] = !data[rowIndex].isSelected;
	},

	resetSelections: function(){
		//	summary
		//	Unselects all data objects.
		this.store.forEach(function(element){
			element.isSelected = false;
		});
		this.selectedColumnIndex = -1;
	},

	onSelectCell: function(/* HTMLEvent */e){
		this.selectedColumnIndex = -1;
		var cell = dvUtil.getParentByType(e.target,"td");
		if (cell) {
			var row = dvUtil.getParentByType(cell,"tr");
			var index = 0;
			for (var i = 0; i < row.cells.length; i++) {
				var c = row.cells[i];
				if (c === cell) {
					this.selectedColumnIndex = index;
					break;
				} else {
					index++;
				}
			}
		}
	},

	renderSelections: function(){
		//	summary
		//	Render all selected objects using CSS.
		var body=this.tableNode.tBodies[0];
		
		// clear old selection
		dojo.query("tr.selected", body).style({
			color: this.dataViewerStyle.tableFontColor
		});
		
		for(var i=0, l=body.rows.length; i<l; i++){
			dojo.toggleClass(body.rows[i], this.rowSelectedClass, this.isRowSelected(body.rows[i], i));
		}
		var ihc = this._getInvertHighContrastColor(this.dataViewerStyle.highlightWithColor);
		dojo.query("tr.selected, tr.highlighted", body).style({
			color: ihc
		});
	},

	//	standard events
	onSelect: function(/* HTMLEvent */e){
		//	summary
		//	Handles the onclick event of any element.
		var row = dvUtil.getParentByType(e.target,"tr");

		this.resetSelections();
		this.toggleSelectionByRow(row);
		this.renderSelections();

		if (this.allowCellSelection) {
			this.onSelectCell(e);
		}
		this.onDataSelected(this.getDataByRow(row), this.selectedColumnIndex);
	},
	
	// Get complementart color
	// e.g. White -> black
	// color {String | Object}
	_getInvertHighContrastColor: function(color) {
		var c = new dojo.Color();
		c.setColor(color || "#000");
		var hsl = c.toHsl();
		hsl.l = (hsl.l >= 50) ? 0 : 100;
		return dojox.color.fromHsl(hsl).toCss();
	},
	
	onRowCellPress: function(event){
		if (event.altKey) { return; }
		var dk = dojo.keys;
		var key = event.charOrCode;
		
		if (!dojo.isString(key)) {	// handle non-printables (arrow keys)
			var map = this._keyBodyHandlerMap;
			if (!map) {
				// setup table mapping keys to events
				map = {};
				map[dk.ENTER] = "onSelect";
				map[this.isLeftToRight() ? dk.LEFT_ARROW : dk.RIGHT_ARROW] = "_onLeftArrow";
				map[this.isLeftToRight() ? dk.RIGHT_ARROW : dk.LEFT_ARROW] = "_onRightArrow";
				map[dk.UP_ARROW]="_onUpArrow";
				map[dk.DOWN_ARROW]="_onDownArrow";
				this._keyBodyHandlerMap = map;
			}
			if (this._keyBodyHandlerMap[key]) {
				this[this._keyBodyHandlerMap[key]](event);
				dojo.stopEvent(event);
			}
		}
    },
    
    onHeaderCellPress: function(event){
		if (event.altKey) { return; }
		var dk = dojo.keys;
		var key = event.charOrCode;
		
		if (!dojo.isString(key)) {	// handle non-printables (arrow keys) 
			var map = this._keyHeaderHandlerMap;
			if (!map) {
				// setup table mapping keys to events
				map = {};
				map[dk.ENTER] = "onSort";
				map[dk.LEFT_ARROW] = "_onLeftArrow";
				map[dk.RIGHT_ARROW] = "_onRightArrow";
				this._keyHeaderHandlerMap = map;
			}
			if (this._keyHeaderHandlerMap[key]) {
				this[this._keyHeaderHandlerMap[key]](event);
				dojo.stopEvent(event);
			}
		}
    },
	
	_onDownArrow: function(e) {
		var currentNode = e.target.parentNode;
		var row = currentNode.parentNode;
			nextRow = row.nextSibling;
		if (nextRow) {
			var index = dvUtil.getNodeIndex(currentNode);
			this._changeFocus(currentNode, dojo.query(":nth-child(" + (index + 1) + ")", nextRow)[0]);
		}
	},
	
	_onUpArrow: function(e) {
		var currentNode = e.target.parentNode;
		var row = currentNode.parentNode;
			previousRow = row.previousSibling;
		if (previousRow) {
			var index = dvUtil.getNodeIndex(currentNode);
			this._changeFocus(currentNode, dojo.query(":nth-child(" + (index + 1) + ")", previousRow)[0]);
		}
	},
	
	_onRightArrow: function(e) {
		var currentNode = e.target.parentNode;
		var nextSlibing = currentNode.nextSibling;
		if (nextSlibing) {
			this._changeFocus(currentNode, nextSlibing);
		}
	},
	
	_onLeftArrow: function(e) {
		var currentNode = e.target.parentNode;
		var previousSibling = currentNode.previousSibling;
		if (previousSibling) {
			this._changeFocus(currentNode, previousSibling);
		}
	},
	
	_changeFocus: function(oldFocusNode, newFocusNode) {
		oldFocusNode = dojo.query("span", oldFocusNode)[0] || oldFocusNode;
		newFocusNode = dojo.query("span", newFocusNode)[0] || newFocusNode;
		
		if (oldFocusNode !== newFocusNode) {
			dojo.attr(oldFocusNode, "tabindex", "-1");
			dijit.setWaiState(oldFocusNode, "selected", false);
			
			dojo.attr(newFocusNode, "tabindex", "0");
			dijit.setWaiState(newFocusNode, "selected", true);
			dijit.focus(newFocusNode);
		}
	}
});
})();
